/*
 * Citadel/UX  
 *
 * citadel.c - Main source file.
 *
 */

#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <pwd.h>
#include <setjmp.h>
#include <termio.h>
#include "citadel.h"
#include "axdefs.h"

struct march {
	struct march *next;
	char march_name[32];
	};

#define IFEXPERT if (userflags&US_EXPERT)
#define IFNEXPERT if ((userflags&US_EXPERT)==0)
#define IFAIDE if (axlevel>=6)
#define IFNAIDE if (axlevel<6)

struct march *march = NULL;
struct passwd *getpwuid();
struct passwd *getpwnam();
char *getenv();
long atol();
long lseek();
long finduser();

/* globals legitimately associated with the client */
char temp[16];				/* Name of general temp file */
char temp2[16];				/* Name of general temp file */
char tempdir[16];			/* Name of general temp dir */
int editor_pid = (-1);
char fullname[32];
jmp_buf nextbuf;
struct serv_info serv_info;		/* Info on the server connected */
int screenwidth;
int screenheight;
unsigned room_flags;
char room_name[32];
char ugname[20];
long uglsn;				/* holds <u>ngoto info */
char is_mail = 0;			/* nonzero when we're in a mail room */
char axlevel = 0;			/* access level */
char is_room_aide = 0;			/* boolean flag, 1 if room aide */
int timescalled;
int posted;
unsigned userflags;
long eternal = 0L;			/* user number */
char newnow;
long highest_msg_read;			/* used for <A>bandon room cmd */
long maxmsgnum;				/* used for <G>oto */
char sigcaught = 0;
char have_xterm = 0;			/* are we running on an xterm? */

extern char server_is_local;		/* defined in ipc module */

/*
 * here is our 'clean up gracefully and exit' routine
 */
void logoff(code)
int code; {
	char delcmd[256];
	if (editor_pid>0) {		/* kill the editor if it's running */
		kill(editor_pid,SIGHUP);
		}

/* shut down the server... but not if the logoff code is 3, because
 * that means we're exiting because we already lost the server
 */
	if (code!=3) serv_puts("QUIT");

/*
 * now clean up various things
 */
	sprintf(delcmd,"exec rm -fr /tmp/cit?%d 2>/dev/null",getpid());
	system(delcmd);			/* delete temp files */

	while (wait(NULL)!=(-1)) ;	/* wait for child processes */
	sttybbs(SB_RESTORE);		/* return the old terminal settings */
	exit(code);			/* exit with the proper exit code */
	}



/*
 * We handle "next" and "stop" much differently than in earlier versions.
 * The signal catching routine simply sets a flag and returns.
 */
void sighandler(which_sig)
int which_sig; {
	signal(SIGINT,SIG_IGN);
	signal(SIGQUIT,SIG_IGN);
	sigcaught = which_sig;
	return;
	}


/*
 * signal catching function for hangups...
 */
void dropcarr() {
	logoff(SIGHUP);
	}



/* general purpose routines */

void formout(name) /* display a file */
char name[];
	{
	char cmd[256];
	sprintf(cmd,"MESG %s",name);
	serv_puts(cmd);
	serv_gets(cmd);
	if (cmd[0]!='1') {
		printf("%s\n",&cmd[4]);
		return;
		}
	fmout(screenwidth,NULL,
		((userflags & US_PAGINATOR) ? 1 : 0),
		screenheight,1,1);
	}


userlist() { 
	char buf[256];
	char fl[256];
	int e;
	int lines_printed = 2;

	serv_puts("LIST");
	serv_gets(buf);
	if (buf[0]!='1') {
		printf("%s\n",&buf[4]);
		return;
		}
	sigcaught = 0;
	sttybbs(SB_YES_INTR);
	printf("       User Name           Num  L  LastCall  Calls Posts\n");
	printf("------------------------- ----- - ---------- ----- -----\n");
	while (serv_gets(buf), strcmp(buf,"000")) {
		if (sigcaught == 0) {
			extract(fl,buf,0);
			printf("%-25s ",fl);
			printf("%5ld %d ",extract_long(buf,2),
				extract_int(buf,1));
			printf("%10s ","");
			printf("%5ld %5ld\n",extract_long(buf,4),extract_long(buf,5));
			++lines_printed;

			if ((userflags & US_PAGINATOR)
	   		&&(lines_printed > (screenheight-2))) {
				hit_any_key();
				lines_printed = 0;
				}
			}
		}
	sttybbs(SB_NO_INTR);
	printf("\n");
	}


/*
 * get info about the server we've connected to
 */
void get_serv_info() {
	char buf[256];
	int a;
	
	serv_puts("INFO");
	serv_gets(buf);
	if (buf[0]!='1') return;

	a = 0;
	while(serv_gets(buf), strcmp(buf,"000")) {
	    switch(a) {
		case 0:		serv_info.serv_pid = atoi(buf);
				break;
		case 1:		strcpy(serv_info.serv_nodename,buf);
				break;
		case 2:		strcpy(serv_info.serv_humannode,buf);
				break;
		case 3:		strcpy(serv_info.serv_fqdn,buf);
				break;
		case 4:		strcpy(serv_info.serv_software,buf);
				break;
		case 5:		serv_info.serv_rev_level = atoi(buf);
				break;
		case 6:		strcpy(serv_info.serv_bbs_city,buf);
				break;
		case 7:		strcpy(serv_info.serv_sysadm,buf);
				break;
		case 9:		strcpy(serv_info.serv_moreprompt,buf);
				break;
		}
	    ++a;
	    }
	}

/*
 * grab assorted info about the user...
 */
void load_user_info(params)
char *params; {
	extract(fullname,params,0);
	axlevel = extract_int(params,1);
	timescalled = extract_int(params,2);
	posted = extract_int(params,3);
	userflags = extract_int(params,4);
	eternal = extract_long(params,5);
	}


/*
 * remove a room from the march list
 */
void remove_march(aaa)
char *aaa; {
	struct march *mptr,*mptr2;

	if (march==NULL) return;

	if (!strucmp(march->march_name,aaa)) {
		mptr = march->next;
		free(march);
		march = mptr;
		return;
		}

	mptr2 = march;
	for (mptr=march; mptr!=NULL; mptr=mptr->next) {
		if (!strucmp(mptr->march_name,aaa)) {
			mptr2->next = mptr->next;
			free(mptr);
			mptr=mptr2;
			}
		else {
			mptr2=mptr;
			}
		}
	}


/*
 * main
 */
main(argc,argv)
int argc;
char *argv[]; {
int a,b,c,d,e,f,file,file2,mcmd,termn8;
FILE *fp;
long aa,cc;
char aaa[100],bbb[100],ccc[100],eee[100];	/* general purpose variables */
char argbuf[32];				/* command line buf */

#ifdef TIOCGWINSZ				/* for X window sizing */
	struct {
		unsigned short height;		/* rows */
		unsigned short width;		/* columns */
		unsigned short xpixels;
		unsigned short ypixels;		/* pixels */
	} xwinsz;
#endif


sttybbs(SB_SAVE);		/* Store the old terminal parameters */
sttybbs(SB_NO_INTR);		/* Install the new ones */
signal(SIGINT,SIG_IGN);
signal(SIGQUIT,SIG_IGN);
signal(SIGHUP,dropcarr);	/* Cleanup gracefully if carrier is dropped */
signal(SIGTERM,dropcarr);	/* Cleanup gracefully if terminated */

printf("Attaching to server...\r");
fflush(stdout);
attach_to_server(argc,argv);
serv_gets(aaa);
if (aaa[0]!='2') {
	printf("%s\n",&aaa[4]);
	logoff(atoi(aaa));
	}
get_serv_info();

printf("%-22s\n%s\n%s\n",serv_info.serv_software,serv_info.serv_humannode,
	serv_info.serv_bbs_city);
screenwidth = 80;		/* default screen dimensions */
screenheight = 24;

printf(" pause    next    stop\n");
printf(" ctrl-s  ctrl-o  ctrl-c\n\n");
formout("hello");		/* print the opening greeting */
printf("\n");

	/* if we're not the login shell, try auto-login */
if (getppid()!=1) {
	serv_puts("AUTO");
	serv_gets(aaa);
	if (aaa[0]=='2') {
		load_user_info(&aaa[4]);
		goto PWOK;
		}
	}

GSTA:	termn8=0; newnow=0;
	do {
		newprompt("Enter your name: ",fullname,29);
		strproc(fullname); 
		if (!strucmp(fullname,"new")) {		/* just in case */
		   printf("Please enter the name you wish to log in with.\n");
		   }
		} while( 
			(!strucmp(fullname,"bbs"))
			|| (!strucmp(fullname,"new"))
			|| (strlen(fullname)==0) );

	if (!strucmp(fullname,"off")) {
		mcmd=29;
		goto TERMN8;
		}

	/* sign on to the server */
	sprintf(aaa,"USER %s",fullname);
	serv_puts(aaa);
	serv_gets(aaa);
	if (aaa[0]!='3') goto NEWUSR;

	/* password authentication */
	newprompt("\rPlease enter your password: ",eee,-19);
	strproc(eee);
	sprintf(aaa,"PASS %s",eee);
	serv_puts(aaa);
	serv_gets(aaa);
	if (aaa[0]=='2') {
		load_user_info(&aaa[4]);
		goto PWOK;
		}
	
	printf("<< wrong password >>\n");
	goto GSTA;

NEWUSR:	printf("No record. Enter as new user? ");
	if (yesno()==0) goto GSTA;

	sprintf(aaa,"NEWU %s",fullname);
	serv_puts(aaa);
	serv_gets(aaa);
	if (aaa[0]!='2') {
		printf("%s\n",aaa);
		goto GSTA;
		}
	load_user_info(&aaa[4]);

	IFNEXPERT formout("changepw");
	newprompt("Please enter a password: ",eee,19);
	strproc(eee);
	sprintf(aaa,"SETP %s",eee);
	serv_puts(aaa);
	serv_gets(aaa);
	if (aaa[0]!='2') {
		printf("%s\n",aaa);
		interr(atoi(aaa));
		}
	newnow=1;

	enter_config(1);
	

PWOK:	printf("%s\nAccess level: %d (%s)\nUser #%ld / Call #%d\n",
		fullname,axlevel,axdefs[axlevel],
		eternal,timescalled);

	serv_puts("CHEK");
	serv_gets(aaa);
	if (aaa[0]=='2') {
		b = extract_int(&aaa[4],0);
		if (b==1) printf("*** You have a new private message in Mail>\n");
		if (b>1)  printf("*** You have %d new private messages in Mail>\n",b);

		if ((axlevel>=6) && (extract_int(&aaa[4],2)>0)) {
			printf("*** Users need validation\n");
			}

		if (extract_int(&aaa[4],1)>0) {
			printf("*** Please register.\n");
			formout("register");
			entregis();
			}
		}

	/* Make up some temporary filenames for use in various parts of the
	 * program.  Don't mess with these once they've been set, because we
	 * "rm -fr /tmp/citC..." and such later on in the program and we don't
	 * want to delete something that we didn't create. */
	sprintf(temp,"/tmp/citA%d",getpid());
	sprintf(temp2,"/tmp/citB%d",getpid());
	sprintf(tempdir,"/tmp/citC%d",getpid());

	/* Get screen dimensions.  First we go to a default of 80x24.  Then
	 * we try to get the user's actual screen dimensions off the server.
	 * However, if we're running on an xterm, all this stuff is
	 * irrelevant because we're going to dynamically size the screen
	 * during the session.
	 */
	screenwidth = 80;
	screenheight = 24;
	serv_puts("GETU");
	serv_gets(aaa);
	if (aaa[0]=='2') {
		screenwidth = extract_int(&aaa[4],0);
		screenheight = extract_int(&aaa[4],1);
		}
	if (getenv("TERM")!=NULL) if (!strcmp(getenv("TERM"),"xterm")) {
		have_xterm = 1;
		}

	/* Enter the lobby */
	dotgoto("_BASEROOM_",1);

/* Main loop for the system... user is logged in. */
	strcpy(ugname,"");
	uglsn = 0L;

	if (newnow==1)	readmsgs(3,1);
		else	readmsgs(1,1);

do {	/* MAIN LOOP OF PROGRAM */
	signal(SIGINT,SIG_IGN);
	signal(SIGQUIT,SIG_IGN);
	mcmd=getcmd(argbuf);

#ifdef TIOCGWINSZ
	if (have_xterm) {	/* dynamically size screen if on an xterm */
		if ( ioctl(0, TIOCGWINSZ, &xwinsz) == 0 ) {
			if (xwinsz.height) screenheight = (int) xwinsz.height;
			if (xwinsz.width) screenwidth = (int) xwinsz.width;
			}
		}
#endif

	if (termn8==0) switch(mcmd) {
	   case 1:	formout("help");
			break;
	   case 4:	entmsg(0,0);
			break;
	   case 36:	entmsg(0,1);
			break;
	   case 46:	entmsg(0,2);
			break;
	   case 5:	updatels();
			gotonext();
			break;
	   case 47:	updatelsa();
			gotonext();
			break;
	   case 20:	updatels();
	   case 52:	dotgoto(argbuf,0);
			break;
	   case 10:	readmsgs(0,1);
			break;
	   case 9:	readmsgs(3,1);
			break;
	   case 13:	readmsgs(1,1);
			break;
	   case 11:	readmsgs(0,(-1));
			break;
	   case 12:	readmsgs(2,(-1));
			break;
	   case 7:	forget();	break;
	   case 18:	subshell();	break;
	   case 38:	updatels();
			entroom();	break;
	   case 22:	killroom();	break;
	   case 32:	userlist();	break;
	   case 27:	invite();	break;
	   case 28:	kickout();	break;
	   case 23:	editthisroom();	break;
	   case 14:	roomdir();	break;
	   case 33:	download(0);	break;
	   case 34:	download(1);	break;
	   case 31:	download(2);	break;
	   case 43:	download(3);	break;
	   case 45:	download(4);	break;
	   case 39:	upload(0);	break;
	   case 40:	upload(1);	break;
	   case 42:	upload(2);	break;
	   case 44:	upload(3);	break;
	   case 16:	ungoto();	break;
	   case 24:	whoknows();	break;
	   case 26:	validate();	break;
	   case 29:	updatels();
			termn8=1;
			break;
	   case 30:	updatels();
			termn8=1;
			break;
	   case 48:	enterinfo();
			break;
	   case 49:	readinfo();
			break;

case 35:
	IFNEXPERT formout("changepw");
	newprompt("Enter new password: ",ccc,19);
	sprintf(aaa,"SETP %s",ccc);
	serv_puts(aaa);
	serv_gets(aaa);
	printf("%s\n",&aaa[4]);
	break;

case 21:
	if (argbuf[0]==0) strcpy(aaa,"?");
	display_help(argbuf);
	break;

case 41:
	formout("register");
	entregis();
	break;

case 15:
	printf("Are you sure (y/n)? ");
	if (yesno()==1) {
		updatels();
		a=0;
		termn8=1;
		}
	break;

case 6:
	gotonext();
	break;

case 3: if (server_is_local) {
		sttybbs(SB_RESTORE);
		sprintf(aaa,"exec ./chat %s",fullname);
		system(aaa);
		sttybbs(SB_NO_INTR);
		}
	else {
		printf("*** Can't chat when server is not local.\n");
		}
	break;

case 2: if (server_is_local) {
		sttybbs(SB_RESTORE);
sprintf(aaa,"USERNAME=\042%s\042; export USERNAME; exec ./subsystem %ld %d %d",
			fullname,
			eternal,screenwidth,axlevel);
		system(aaa);
		sttybbs(SB_NO_INTR);
		}
	else {
		printf("*** Can't run doors when server is not local.\n");
		}
	break;

case 17:
	printf("Session ID        User Name                  Room\n");
	printf("---------- ------------------------- --------------------\n");
	serv_puts("RWHO");
	serv_gets(aaa);
	if (aaa[0]=='1') {
		while(serv_gets(aaa), strcmp(aaa,"000")) {
			extract(bbb,aaa,1);
			extract(ccc,aaa,2);
			printf("%-10d %-25s %-20s\n",
				extract_int(aaa,0),bbb,ccc);
			}
		}
	break;

case 50:
	enter_config(2);
	break;

case 37:
	enter_config(0);
	break;

case 25:
	edituser();
	break;

case 8:
	knrooms();
	printf("\n");
	break;

case 19:
	listzrooms();
	printf("\n");
	break;

case 51:
	deletefile();
	break;

case 53:
	netsendfile();
	break;

case 54:
	movefile();
	break;

	} /* end switch */
    } while(termn8==0);

TERMN8:	printf("%s logged out.\n",fullname);
	while (march!=NULL) remove_march(march->march_name);
	if (mcmd==30)
		printf("\n\nType 'off' to hang up, or next user...\n");
	sprintf(aaa,"LOUT");
	serv_puts(aaa);
	serv_gets(aaa);
	if ((mcmd==29)||(mcmd==15)) {
		formout("goodbye");
		logoff(0);
		}
	goto GSTA;

} /* end main() */

/* Goto next room having unread messages.
 * We want to skip over rooms that the user has already been to, and take the
 * user back to the lobby when done.  The room we end up in is placed in
 * newroom - which is set to 0 (the lobby) initially.
 * We start the search in the current room rather than the beginning to prevent
 * two or more concurrent users from dragging each other back to the same room.
 */
gotonext() {
	int a,newroom;
	FILE *fp;
	char buf[256];
	struct march *mptr,*mptr2;
	char next_room[32];

	/* First check to see if the march-mode list is already allocated.
	 * If it is, pop the first room off the list and go there.
	 */

	if (march==NULL) {
		serv_puts("LKRN");
		serv_gets(buf);
		if (buf[0]=='1')
		    while (serv_gets(buf), strcmp(buf,"000")) {
			mptr = (struct march *) malloc(sizeof(struct march));
			mptr->next = NULL;
			extract(mptr->march_name,buf,0);
			if (march==NULL) {
				march = mptr;
				}
			else {
				mptr2 = march;
				while (mptr2->next != NULL)
					mptr2 = mptr2->next;
				mptr2->next = mptr;
				}
			}

/* add _BASEROOM_ to the end of the march list, so the user will end up
 * in the system base room (usually the Lobby>) at the end of the loop
 */
		mptr = (struct march *) malloc(sizeof(struct march));
		mptr->next = NULL;
		strcpy(mptr->march_name,"_BASEROOM_");
		if (march==NULL) {
			march = mptr;
			}
		else {
			mptr2 = march;
			while (mptr2->next != NULL)
				mptr2 = mptr2->next;
			mptr2->next = mptr;
			}
/*
 * ...and remove the room we're currently in, so a <G>oto doesn't make us
 * walk around in circles
 */
		remove_march(room_name);
		}


	if (march!=NULL) {
		strcpy(next_room,march->march_name);
		}
	else {
		strcpy(next_room,"_BASEROOM_");
		}
	dotgoto(next_room,1);
   }

dotgoto(towhere,display_name)
char *towhere;
int display_name; {
	int a,b,c,d,file;
	char aaa[256],bbb[256],ccc[256];
	static long ls = 0L;

	/* store ungoto information */
	strcpy(ugname,room_name);
	uglsn = ls;

	/* first try an exact match */
	sprintf(aaa,"GOTO %s",towhere);
	serv_puts(aaa);
	serv_gets(aaa);
	if (!strncmp(aaa,"54",2)) {
		newprompt("Enter room password: ",bbb,9);
		sprintf(aaa,"GOTO %s|%s",towhere,bbb);
		serv_puts(aaa);
		}
	if (!strncmp(aaa,"54",2)) {
		printf("Wrong password.\n");
		return(1);
		}

	if (aaa[0]!='2') {		/* try a partial match */
		strcpy(bbb,"");
		serv_puts("LKRA");
		serv_gets(aaa);
		if (aaa[0]=='1') while (serv_gets(aaa), strcmp(aaa,"000")) {
			extract(ccc,aaa,0);
			if (pattern(ccc,towhere)>=0) {
				strcpy(bbb,ccc);
				}
			}
		if (strlen(bbb)==0) {
			printf("No room '%s'.\n",towhere);
			return(1);
			}
		sprintf(aaa,"GOTO %s",bbb);
		serv_puts(aaa);
		serv_gets(aaa);
		}

	if (aaa[0]!='2') {
		printf("%s\n",aaa);
		return(1);
		}

	extract(room_name,&aaa[4],0);
	room_flags = extract_int(&aaa[4],4);

	remove_march(room_name);
	if (!strucmp(towhere,"_BASEROOM_")) remove_march(towhere);
	if (display_name) printf("%s - ",room_name);
	printf("%d new of %d messages.\n",
		extract_int(&aaa[4],1),
		extract_int(&aaa[4],2));
	highest_msg_read = extract_int(&aaa[4],6);
	maxmsgnum = extract_int(&aaa[4],5);
	is_mail = (char) extract_int(&aaa[4],7);
	is_room_aide = (char) extract_int(&aaa[4],8);
	ls = extract_long(&aaa[4],6);


	/* read info file if necessary */
	if (extract_int(&aaa[4],3) > 0) readinfo();
	}

