#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include "citadel.h"
#ifdef USE_SYSLOG
#include <syslog.h>
#endif

extern int errno;

struct usersupp usersupp;
struct quickroom quickroom;
struct fullroom fullroom;
struct msgmain msgmain;
char curr_user[32];
int curr_rm = (-1);
int logged_in = 0;
struct config config;
char temp[32];
int cs_pos;
struct citsession citsession;
int nologin;
FILE *download_fp = NULL;
FILE *upload_fp = NULL;
FILE *monitor = NULL;

/*
 * record an entry in the call log
 */
void rec_log(lrtype,name)
unsigned lrtype;
char *name; {
	struct calllog record;
	int file,file2,a,b;


	time(&record.CLtime);
	record.CLflags = lrtype;
	strcpy(record.CLfullname,name);
	file=open("calllog.pos",O_RDWR);

	lock_file(file,0L,2L);
	read(file,&a,sizeof(int));
	b=a; ++a; if (a>=CALLLOG) a=0;
	lseek(file,0L,0);
	write(file,&a,sizeof(int));

	file2=open("calllog",O_RDWR);
	lseek(file2,(long)(b*sizeof(struct calllog)),0);
	write(file2,(char *)&record,sizeof(struct calllog));
	close(file2);

	unlock_file(file,0L,2L);
	close(file);
	}


/*
 * cleanup()  -  close up various things
 */
void cleanup(exit_code) {
	FILE *fp;

	getroom(&quickroom,(-1));
	free_ucache();
	fp=fopen("sessions","r+");
	citsession.cs_pid = 0;
	fseek(fp,( ((long)cs_pos)*((long)sizeof(struct citsession)) ),0);
	fwrite((char *)&citsession,sizeof(struct citsession),1,fp);
	fclose(fp);
	rec_log(CL_TERMINATE,curr_user);
	unlink(temp);
#ifdef LOGFILE
	fprintf(stderr,"citserver[%2d]: ended.\n",cs_pos);
#endif
#ifdef MONITOR
	fprintf(monitor,"\033[%d;5H%2d : %30s : %-30s",
		cs_pos+1, cs_pos, "", "");
	fflush(monitor);
#endif
#ifdef USE_SYSLOG
	syslog(LOG_NOTICE,"session %d ended - exit code %d",cs_pos,exit_code);
	closelog();
#endif
	exit(exit_code);
	}

/*
 * set_wtmpsupp()  -  alter the session listing
 */
void set_wtmpsupp(newtext)
char *newtext; {
	FILE *fp;
	fp=fopen("sessions","r+");
	strcpy(citsession.cs_room,newtext);
	time(&citsession.cs_lastupdt);
	fseek(fp,( ((long)cs_pos)*((long)sizeof(struct citsession)) ),0);
	fwrite((char *)&citsession,sizeof(struct citsession),1,fp);
	fclose(fp);
#ifdef MONITOR
	fprintf(monitor,"\033[%d;5H%2d : %30s : %-30s",
		cs_pos+1, cs_pos, curr_user, newtext);
	fflush(monitor);
#endif
	}


/*
 * init_session()  -  start up various things  (returns session number)
 */
int init_session() {
	FILE *fp,*pr;
	struct citsession citsession;
	int a,sessions;
	char buf[256];

	lock_msgmain();

	fp=fopen("sessions","r+");
	if (fp==NULL) {
		fp=fopen("sessions","w");
		fclose(fp);
		fp=fopen("sessions","r+");
		}
	if (fp==NULL) {
#ifdef LOGFILE
		fprintf(stderr,"citserver: cannot setup session rec\n");
#endif
		exit(errno);
		}	
	cs_pos = (-1);
	a = 0;
	sessions = 0;
	while (fread((char *)&citsession,sizeof(struct citsession),1,fp)>0) {

/* if we have the /proc filesystem we can locate zombie server processes */
#ifdef HAVE_PROC_FS
		if (citsession.cs_pid != 0) {
			sprintf(buf,"/proc/%d/cmdline",citsession.cs_pid);
			pr=fopen(buf,"r");
			if (pr==NULL) {
				kill(citsession.cs_pid,15);
				kill(citsession.cs_pid,9);
				citsession.cs_pid = 0;
				}
			else {
				fclose(pr);
				}
			}
#endif

		if (citsession.cs_pid > 0) {
			++sessions;
			}
		else {
			if (cs_pos < 0) cs_pos = a;
			}
		++a;
		}
	if (cs_pos < 0) cs_pos = a;
	fseek(fp,(((long)cs_pos)*((long)sizeof(struct citsession))),0);
	citsession.cs_pid = getpid();
	time(&citsession.cs_lastupdt);
	strcpy(citsession.cs_name,"(not logged in)");
	strcpy(citsession.cs_room,"(no room)");
	fwrite((char *)&citsession,sizeof(struct citsession),1,fp);
	fclose(fp);
	unlock_msgmain();

	for (a=0; a<=20; ++a) signal(a,cleanup);
	signal(SIGINT,SIG_IGN);
	signal(SIGQUIT,SIG_IGN);
	rec_log(CL_INOTHER,"");


#ifdef USE_SYSLOG
	syslog(LOG_NOTICE,"session %d started",cs_pos);
#endif

#ifdef MONITOR
	fprintf(monitor,"\033[%d;5H%2d : %30s : %-30s",
		cs_pos+1, cs_pos, "(not logged in)", "(no room)");
	fflush(monitor);
#endif

	return(sessions);
	}


/*
 * cmd_info()  -  tell the client about this server
 */
void cmd_info() {
	printf("%d Server info:\n",LISTING_FOLLOWS);
	printf("%d\n",getpid());
	printf("%s\n",config.c_nodename);
	printf("%s\n",config.c_humannode);
	printf("%s\n",config.c_fqdn);
	printf("%s\n",CITADEL);
	printf("%d\n",REV_LEVEL);
	printf("%s\n",config.c_bbs_city);
	printf("%s\n",config.c_sysadm);
	printf("%d\n",SERVER_TYPE);
	printf("%s\n",config.c_moreprompt);
	printf("000\n");
	}


/*
 * display system messages or help
 */
void cmd_mesg(mname)
char *mname; {
	FILE *mfp;
	char buf[256];

	sprintf(buf,"./messages/%s",mname);
	mfp = fopen(buf,"r");
	if (mfp==NULL) {
		sprintf(buf,"./help/%s",mname);
		mfp = fopen(buf,"r");
		}
	if (mfp==NULL) {
		printf("%d '%s' not found.\n",ERROR,mname);
		return;
		}
	printf("%d %s\n",LISTING_FOLLOWS,buf);

	while (fgets(buf,255,mfp)!=NULL) {
		buf[strlen(buf)-1] = 0;
		do_help_subst(buf);
		printf("%s\n",buf);
		}


	fclose(mfp);
	printf("000\n");
	}


/*
 * who's online
 */
void cmd_rwho() {
	FILE *sess;
	struct citsession cs_buf;

	sess=fopen("sessions","r");
	if (sess==NULL) exit(0);

	printf("%d\n",LISTING_FOLLOWS);
	while (fread((char *)&cs_buf,sizeof(struct citsession),1,sess)==1) {
		if (cs_buf.cs_pid != 0) {
			printf("%d|%s|%s\n",
				cs_buf.cs_pid,cs_buf.cs_name,cs_buf.cs_room);
			}
		}
	fclose(sess);
	printf("000\n");
	}


/* 
 * get the paginator prompt
 */
void cmd_more() {
	printf("%d %s\n",OK,config.c_moreprompt);
	}

/*
 * echo 
 */
void cmd_echo(etext)
char *etext; {
	printf("%d %s\n",OK,etext);
	}



/*
 * main loop 
 */
main() {
	char cmdbuf[256];
	int fd;
	int pid;
	int session_num;

#ifdef LOGFILE
	freopen(LOGFILE,"w",stderr);
#else
	freopen("/dev/null","w",stderr);
#endif
#ifdef MONITOR
	monitor = fopen(MONITOR,"w");
#endif
#ifdef USE_SYSLOG
	openlog("citserver",LOG_PID,LOG_USER);
#endif

	get_config();
	strcpy(curr_user,"");
	sprintf(temp,"/tmp/csv%05d",getpid());
	session_num = init_session();
	nologin = 0;
	if ((config.c_maxsessions > 0)&&(session_num >= config.c_maxsessions))
		nologin = 1;

	if (nologin==1) {
	   printf("%d %s: Too many users are already online (maximum is %d)\n",
		ERROR+MAX_SESSIONS_EXCEEDED,
		config.c_nodename,config.c_maxsessions);
		}
	else {
	   printf("%d %s Citadel/UX server ready.\n",OK,config.c_nodename);
		}

	pid=getpid();
#ifdef LOGFILE
	fprintf(stderr,"citserver[%2d]: started.  pid=%d\n",cs_pos,pid);
#endif

	do {
		fflush(stdout);
		alarm(config.c_sleeping);
		if (fgets(cmdbuf,256,stdin) == NULL) cleanup(EXIT_NULL);
		alarm(0);
		cmdbuf[strlen(cmdbuf)-1] = 0;
#ifdef LOGFILE
		fprintf(stderr,"citserver[%2d]: %s\n",cs_pos,cmdbuf);
#endif

/*
 * This loop recognizes all server commands.
 */

		if (!struncmp(cmdbuf,"NOOP",4)) {
			printf("%d ok\n",OK);
			}

		else if (!struncmp(cmdbuf,"QUIT",4)) {
			printf("%d Goodbye.\n",OK);
			}

		else if (!struncmp(cmdbuf,"LOUT",4)) {
			if (logged_in) logout();
			printf("%d logged out.\n",OK);
			}

		else if (!struncmp(cmdbuf,"USER",4)) {
			cmd_user(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"PASS",4)) {
			cmd_pass(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"NEWU",4)) {
			cmd_newu(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"SETP",4)) {
			cmd_setp(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"AUTO",4)) {
			cmd_auto();
			}

		else if (!struncmp(cmdbuf,"LRMS",4)) {
			cmd_lrms();
			}

		else if (!struncmp(cmdbuf,"LKRA",4)) {
			cmd_lkra();
			}

		else if (!struncmp(cmdbuf,"LKRN",4)) {
			cmd_lkrn();
			}

		else if (!struncmp(cmdbuf,"LKRO",4)) {
			cmd_lkro();
			}

		else if (!struncmp(cmdbuf,"LZRM",4)) {
			cmd_lzrm();
			}

		else if (!struncmp(cmdbuf,"GETU",4)) {
			cmd_getu();
			}

		else if (!struncmp(cmdbuf,"SETU",4)) {
			cmd_setu(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"GOTO",4)) {
			cmd_goto(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"MSGS",4)) {
			cmd_msgs(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"WHOK",4)) {
			cmd_whok();
			}

		else if (!struncmp(cmdbuf,"RDIR",4)) {
			cmd_rdir();
			}

		else if (!struncmp(cmdbuf,"MSG0",4)) {
			cmd_msg0(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"INFO",4)) {
			cmd_info();
			}

		else if (!struncmp(cmdbuf,"SLRP",4)) {
			cmd_slrp(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"INVT",4)) {
			cmd_invt_kick(&cmdbuf[5],1);
			}

		else if (!struncmp(cmdbuf,"KICK",4)) {
			cmd_invt_kick(&cmdbuf[5],0);
			}

		else if (!struncmp(cmdbuf,"GETR",4)) {
			cmd_getr();
			}

		else if (!struncmp(cmdbuf,"SETR",4)) {
			cmd_setr(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"GETA",4)) {
			cmd_geta();
			}

		else if (!struncmp(cmdbuf,"SETA",4)) {
			cmd_seta(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"ENT0",4)) {
			cmd_ent0(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"RINF",4)) {
			cmd_rinf();
			}

		else if (!struncmp(cmdbuf,"DELE",4)) {
			cmd_dele(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"KILL",4)) {
			cmd_kill(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"CRE8",4)) {
			cmd_cre8(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"MOVE",4)) {
			cmd_move(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"FORG",4)) {
			cmd_forg();
			}

		else if (!struncmp(cmdbuf,"MESG",4)) {
			cmd_mesg(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"GNUR",4)) {
			cmd_gnur();
			}

		else if (!struncmp(cmdbuf,"GREG",4)) {
			cmd_greg(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"VALI",4)) {
			cmd_vali(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"EINF",4)) {
			cmd_einf(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"LIST",4)) {
			cmd_list();
			}

		else if (!struncmp(cmdbuf,"REGI",4)) {
			cmd_regi();
			}

		else if (!struncmp(cmdbuf,"CHEK",4)) {
			cmd_chek();
			}

		else if (!struncmp(cmdbuf,"DELF",4)) {
			cmd_delf(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"MOVF",4)) {
			cmd_movf(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"NETF",4)) {
			cmd_netf(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"RWHO",4)) {
			cmd_rwho(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"OPEN",4)) {
			cmd_open(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"CLOS",4)) {
			cmd_clos();
			}

		else if (!struncmp(cmdbuf,"UOPN",4)) {
			cmd_uopn(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"UCLS",4)) {
			cmd_ucls(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"READ",4)) {
			cmd_read(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"WRIT",4)) {
			cmd_writ(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"QUSR",4)) {
			cmd_qusr(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"ECHO",4)) {
			cmd_echo(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"OIMG",4)) {
			cmd_oimg(&cmdbuf[5]);
			}

		else if (!struncmp(cmdbuf,"MORE",4)) {
			cmd_more();
			}

		else {
			printf("%d Command not supported.\n",ERROR);
			}

		} while(struncmp(cmdbuf,"QUIT",4));

	if (logged_in) logout();
	cleanup(EXIT_NORMAL);
	}
