/*
 * Citadel/UX
 *
 * commands.c - front end for Citadel
 *
 * This version is the traditional command parser for room prompts.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <termio.h>
#include <signal.h>
#include "citadel.h"

#define IFNEXPERT if ((userflags&US_EXPERT)==0)

extern unsigned room_flags;
extern char room_name[];
extern struct serv_info serv_info;
extern char axlevel;
extern char is_room_aide;
extern unsigned userflags;
extern char sigcaught;

/* these variables are local to this module */
char keepalives_enabled = 1;		/* send NOOPs to server when idle */
int total_idle;				/* total keyboard idle time */

void sighandler();

void set_keepalives(s)
int s; {
	keepalives_enabled = (char)s;
	}

void inkey_timeout() {
	char buf[256];

	signal(SIGALRM,SIG_IGN);
	total_idle = total_idle + S_KEEPALIVE;
	if ((total_idle > SLEEPING) && (SLEEPING != 0) && (getppid()==1)) {
		printf("Sleeping? Call again.\n");
		logoff(SIGALRM);
		}
	if (keepalives_enabled) {
		serv_puts("NOOP");
		serv_gets(buf);
		}
	alarm(S_KEEPALIVE);
	signal(SIGALRM,inkey_timeout);
	}


inkey() {		/* get a character from the keyboard, with   */
	int a;		/* the watchdog timer in effect if necessary */
	
	total_idle = 0;
	alarm(S_KEEPALIVE);
	signal(SIGALRM,inkey_timeout);
	do {
		a=getc(stdin);
		if (a==127) a=8;
		if (a>126) a=0;
		if (((a!=4)&&(a!=13)&&(a!=8)&&(a!=NEXT_KEY)&&(a!=STOP_KEY))
			&& ((a<32)||(a>126))) a=0;
		} while(a==0);
	signal(SIGALRM,SIG_IGN);
	alarm(0);
	return(a);
	}


getline(string,lim)	/* Gets a line from the terminal */
char string[]; 		/* Pointer to string buffer */
int lim;		/* Maximum length - if negative, no-show */
{
int a,b; char flag;
	flag=0;
	if (lim<0) { lim=(0-lim); flag=1; }
	strcpy(string,"");
GLA:	a=inkey(); a=(a&127);
	if ((a==8)&&(strlen(string)==0)) goto GLA;
	if ((a!=13)&&(a!=8)&&(strlen(string)==lim)) goto GLA;
	if ((a==8)&&(string[0]!=0)) {
		string[strlen(string)-1]=0;
		putc(8,stdout); putc(32,stdout); putc(8,stdout); goto GLA; }
	if ((a==13)||(a==10)) {
		putc(13,stdout); putc(10,stdout); return(0); }
	if (a<32) a='.';
	b=strlen(string);
	string[b]=a;
	string[b+1]=0;
	if (flag==0) putc(a,stdout);
	if (flag==1) putc('*',stdout);
	goto GLA;
	}

/*
 * strprompt()  -  prompt for a string, print the existing value and
 *                 allow the user to press return to keep it...
 */
void strprompt(prompt,str,len)
char *prompt;
char *str;
int len; {
	char buf[128];
	printf("%s [%s]: ",prompt,str);
	getline(buf,len);
	if (buf[0]!=0) strcpy(str,buf);
	}

/* 
 * intprompt()  -  like strprompt(), except for an integer
 *                 (note that it RETURNS the new value!)
 */
int intprompt(prompt,ival,imin,imax)
char *prompt;
int ival;
int imin;
int imax; {
	char buf[16];
	int i;
	i = ival;
	do {
		sprintf(buf,"%d",i);
		strprompt(prompt,buf,15);
		i=atoi(buf);
		if (i<imin) printf("*** Must be no less than %d.\n",imin);
		if (i>imax) printf("*** Must be no more than %d.\n",imax);
		} while((i<imin)||(i>imax));
	return(i);
	}

/* 
 * newprompt()  -  prompt for a string with no existing value
 *                 (clears out string buffer first)
 */
void newprompt(prompt,str,len)
char *prompt;
char *str;
int len; {
	printf("%s",prompt);
	getline(str,len);
	}


int lkey() {	/* returns a lower case value */
	int a;
	a=inkey();
	if (isupper(a)) a=tolower(a);
	return(a);
	}

/*
 * This routine is called to get and parse a command.
 * It returns the command number.  If the command also requires an
 * argument (like .Goto: or .Help: etc.) it will be placed in strbuf.
 */
int getcmd(strbuf)
char *strbuf; {

	int a,b,c,d;
	strcpy(strbuf,"");

	/* if we're running in idiot mode, display a cute little menu */
	IFNEXPERT formout("mainmenu");

/* now the room prompt... */
printf("\n%s%c ",room_name,room_prompt(room_flags));

 while (1) {
  a=lkey();
  if ((a=='!')&&(axlevel<6)) a=0;
  if (a=='?') a='h';
  switch(a) {
   case 'h':	printf("Help\n");
		return(1);
   case '*':	printf("Doorway\n");
		return(2);
   case 'c':	printf("Chat\n");
		return(3);
   case 'e':	printf("Enter message\n");
		return(DEFAULT_ENTRY);
   case 'g':	printf("Goto\n");
		return(5);
   case 's':	printf("Skip %s\n",room_name);
		return(6);
   case 'z':	printf("Zap (forget) room\n");
		return(7);
   case 'k':	printf("Known rooms\n");
		return(8);
   case 'l':	printf("Last five msgs\n");
		return(9);
   case 'f':	printf("Forward read\n");
		return(10);
   case 'r':	printf("Reverse read\n");
		return(11);
   case 'o':	printf("Old msgs reverse\n");
		return(12);
   case 'n':	printf("New messages\n");
		return(13);
   case 'd':	printf("Directory\n");
		return(14);
   case 't':	printf("Terminate\n");
		return(15);
   case 'u':	printf("Ungoto\n");
		return(16);
   case 'w':	printf("Who is online\n");
		return(17);
   case 'a':	printf("Abandon room, goto...\n");
		return(47);
   case 'x':	printf("toggle eXpert mode\n");
		return(50);
   case '!':	printf("! <shell>\n");
		return(18);
   case '.': printf(".");
	     do {
		b=lkey();
		if ((b=='a')&&(!is_room_aide)) b=0;
		switch(b) {
		   case 'z':	printf("Zapped list\n");
				return(19);
		   case 's':	printf("Skip this room, goto: ");
				getline(strbuf,19);
				return(52);
		   case 'g':	printf("Goto: ");
				getline(strbuf,19);
				return(20);
		   case 'h':	printf("Help: ");
				getline(strbuf,31);
				return(21);

/* start of aide options */
		   case 'a': printf("Aide ");
			     do {
				c=lkey();
if (axlevel<6)
    if ((c!='r')&&(c!='e')&&(c!='w')&&(c!='i')&&(c!='f')&&(c!='?')&&(c!=8))
	c=0;
				switch(c) {
				   case 'k':	printf("Kill this room\n");
						return(22);
				   case 'e':	printf("Edit this room\n");
						return(23);
				   case 'w':	printf("Who knows room\n");
						return(24);
				   case 'u':	printf("User edit\n");
						return(25);
				   case 'v':	printf("Validate new users\n");
						return(26);
				   case 'i':	printf("enter Info file\n");
						return(48);

/* start of room options */
				   case 'r': printf("Room cmd: ");
					     do {
						d=lkey();
						switch(d) {
					case 'i': printf("Invite user\n");
						  return(27);
					case 'k': printf("Kick out user\n");
						  return(28);
						  }
						} while(d!=8);
					     back(10);
					     break;
/* end of room options */

/* start of file options */
				   case 'f': printf("File cmd: ");
					     do {
						d=lkey();
						switch(d) {
					case 'd': printf("Delete file\n");
						  return(51);
					case 's': printf("Send over net\n");
						  return(53);
					case 'm': printf("Move file\n");
						  return(54);
						  }
						} while(d!=8);
					     back(10);
					     break;
/* end of room options */

				   case '?':	printf("(options)\n");
						formout("aideopt");
printf("%s%c .Aide ",room_name,room_prompt(room_flags));
					}
				} while(c!=8);
			     back(5);
			     break;
/* end of aide options */

/* start of terminate options */
		   case 't': printf("Terminate ");
			     do {
				c=lkey();
				switch(c) {
				   case 'q':	printf("and Quit\n");
						return(29);
				   case 's':	printf("and Stay online\n");
						return(30);
					}
				} while(c!=8);
			     back(10);
			     break;
/* end of terminate options */

/* start of read options */
		   case 'r': printf("Read ");
			     do {
				c=lkey();
				switch(c) {
				   case 'f':	printf("File unformatted\n");
						return(31);
				   case 'u':	printf("User listing\n");
						return(32);
				   case 't':	printf("Textfile formatted\n");
						return(33);
				   case 'x':	printf("file using Xmodem\n");
						return(34);
				   case 'y':	printf("file using Ymodem\n");
						return(43);
				   case 'z':	printf("file using Zmodem\n");
						return(45);
				   case 'n':	printf("New messages\n");
						return(13);
				   case 'o':	printf("Old msgs reverse\n");
						return(12);
				   case 'l':	printf("Last five msgs\n");
						return(9);
				   case 'd':	printf("Directory\n");
						return(14);
				   case 'i':	printf("Info file\n");
						return(49);
				   case '?':	printf("(options)\n");
						formout("readopt");
printf("%s%c .Read ",room_name,room_prompt(room_flags));
					}
				} while(c!=8);
			     back(5);
			     break;
/* end of read options */

/* start of enter options */
		   case 'e': printf("Enter ");
			     do {
				c=lkey();
				switch(c) {
				   case 'p':	printf("Password\n");
						return(35);
				   case 'a':	printf("ASCII message\n");
						return(36);
				   case 'c':	printf("Configuration\n");
						return(37);
				   case 'r':	printf("a new Room\n");
						return(38);
				   case 't':	printf("Textfile\n");
						return(39);
				   case 'x':	printf("file using Xmodem\n");
						return(40);
				   case 'y':	printf("file using Ymodem\n");
						return(42);
				   case 'z':	printf("file using Zmodem\n");
						return(44);
				   case 'g':	printf("reGistration\n");
						return(41);
				   case 'm':	printf("Message\n");
						return(4);
				   case 'e':	printf("message with Editor\n");
						return(46);
				   case '?':	printf("(options)\n");
						formout("entopt");
printf("%s%c .Enter ",room_name,room_prompt(room_flags));
					}
				} while(c!=8);
			     back(6);
			     break;
/* end of enter options */

		   case '?':	printf("(options)\n");
				formout("dotopt");
printf("%s%c .",room_name,room_prompt(room_flags));
		   }
		} while(b!=8);
		back(1);
		break;

    }	/* end switch	*/
  }	/* end while	*/
}	/* end getcmd()	*/


/*
 * set tty modes.  commands are:
 * 
 * 0 - set to bbs mode, intr/quit disabled
 * 1 - set to bbs mode, intr/quit enabled
 * 2 - save current settings for later restoral
 * 3 - restore saved settings
 */
sttybbs(cmd) 
int cmd; {
	struct termio live;
	static struct termio saved_settings;

	if ( (cmd == 0) || (cmd == 1) ) {
		ioctl(0,TCGETA,&live);
		live.c_iflag=ISTRIP|IXON|IXANY;
		live.c_oflag=OPOST|ONLCR;
		live.c_lflag=NOFLSH;
		if (cmd==1) live.c_lflag=ISIG|NOFLSH;
		live.c_line=0;

		if (cmd==SB_YES_INTR) {
			live.c_cc[0]=NEXT_KEY;
			live.c_cc[1]=STOP_KEY;
			signal(SIGINT,sighandler);
			signal(SIGQUIT,sighandler);
			}
		else {
			signal(SIGINT,SIG_IGN);
			signal(SIGQUIT,SIG_IGN);
			live.c_cc[0]=(-1);
			live.c_cc[1]=(-1);
			}
		live.c_cc[2]=8;
		live.c_cc[3]=24;
		live.c_cc[4]=1;
		live.c_cc[5]=255;
		live.c_cc[6]=0;
		live.c_cc[7]=0;
		ioctl(0,TCSETAF,&live);
		}
	if (cmd == 2) {
		ioctl(0,TCGETA,&saved_settings);
		}
	if (cmd == 3) {
		ioctl(0,TCSETAF,&saved_settings);
		}
	}


/*
 * display_help()  -  help file viewer
 */
void display_help(name)
char name[]; {
	formout(name);
	}


/*
 * fmout()  -  Citadel text formatter and paginator
 */
int fmout(width,fp,pagin,height,starting_lp,subst)
int width;		/* screen width to use */
FILE *fp;		/* file to read from, or NULL to read from server */
char pagin;		/* nonzero if we should use the paginator */
int height;		/* screen height to use */
int starting_lp;	/* starting value for lines_printed */
char subst;		/* nonzero if we should use hypertext mode */
	{
	int a,b,c,d,real,old;
	char aaa[140],buf2[16];
	char buffer[512];
	int lines_printed;
	int eof_flag = 0;
	
	lines_printed = starting_lp;
	strcpy(aaa,""); old=255;
	strcpy(buffer,"");
	c=1; /* c is the current pos */

	sigcaught = 0;
	sttybbs(1);

FMTA:	while ( (eof_flag==0) && (strlen(buffer)<126) ) {
		if (sigcaught) goto OOPS;
		if (fp!=NULL) { /* read from file */
			if (feof(fp)) eof_flag = 1;
			if (eof_flag==0) {
				a=getc(fp);
				buffer[strlen(buffer)+1] = 0;
				buffer[strlen(buffer)] = a;
				}
			}
		else {		/* read from server */
			d=strlen(buffer);
			serv_gets(&buffer[d]);
while ( (!isspace(buffer[d])) && (isspace(buffer[strlen(buffer)-1])) )
	buffer[strlen(buffer)-1]=0;
			if (!strcmp(&buffer[d],"000")) {
				buffer[d] = 0;
				eof_flag = 1;
				while(isspace(buffer[strlen(buffer)-1]))
					buffer[strlen(buffer)-1] = 0;
				}
			d=strlen(buffer);
			buffer[d] = 10;
			buffer[d+1] = 0;
			}
		}

	buffer[strlen(buffer)+1] = 0;
	a=buffer[0];
	strcpy(buffer,&buffer[1]);
	
	old=real;
	real=a;
	if (a<=0) goto FMTEND;
	
	if ( ((a==13)||(a==10)) && (old!=13) && (old!=10) ) a=32;
	if ( ((old==13)||(old==10)) && (isspace(real)) ) {
		printf("\n");
		++lines_printed;
		lines_printed = checkpagin(lines_printed,pagin,height);
		c=1;
		}
	if (a>126) goto FMTA;

	if (a>32) {
	if ( ((strlen(aaa)+c)>(width-5)) && (strlen(aaa)>(width-5)) )
		{ printf("\n%s",aaa); c=strlen(aaa); aaa[0]=0;
		++lines_printed;
		lines_printed = checkpagin(lines_printed,pagin,height);
		}
	 b=strlen(aaa); aaa[b]=a; aaa[b+1]=0; }
	if (a==32) {
		if ((strlen(aaa)+c)>(width-5)) { 
			printf("\n");
			c=1;
			++lines_printed;
			lines_printed = checkpagin(lines_printed,pagin,height);
			}
		printf("%s ",aaa); ++c; c=c+strlen(aaa);
		strcpy(aaa,"");
		goto FMTA;
		}
	if ((a==13)||(a==10)) {
		printf("%s\n",aaa);
		c=1;
		++lines_printed;
		lines_printed = checkpagin(lines_printed,pagin,height);
		strcpy(aaa,"");
		goto FMTA;
		}
	goto FMTA;

	/* signal caught; drain the server */
OOPS:	do {
		serv_gets(aaa);
		} while(strcmp(aaa,"000"));

FMTEND:	printf("\n");
	return(sigcaught);
}

