#! /bin/sh

#   This is the configuration script for the Citadel/UX Makefile.  Do not attempt
# to edit any of this stuff yourself unless you really know what you are doing!

echo -------- Now generating the sysdep.h header for your system. --------

cp /dev/null sysdep.h



##############################################################################
#                                                                            #
#                  Test to see if we have the syslog facility                #
#                                                                            #
##############################################################################

echo
echo Testing to see if you have the syslog facility available...
cat >testing.c <<!

#include <syslog.h>

main() {
	openlog("logtester",0,LOG_USER);
	syslog(LOG_NOTICE,"hi there!");
	closelog();
	}
!

if cc testing.c -o testing 2>/dev/null; then
	echo ...syslog available.
	echo "#define USE_SYSLOG" >>sysdep.h
else
	echo ...not available.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#                  Test to see if we have the flock() system call            #
#                                                                            #
##############################################################################

echo
echo Testing for flock system call...
cat >testing.c <<!


#include <stdio.h>
#include <fcntl.h>
#include <sys/file.h>

main() {
	int mms_fd;
	int a;

	a=flock(mms_fd,LOCK_EX);
	a=flock(mms_fd,LOCK_UN);
	exit(0);
	}


!

if cc testing.c -o testing 2>/dev/null; then
	echo ...flock available.
	echo "#define HAVE_FLOCK" >>sysdep.h
else
	echo ...not available.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#                  Test for SystemV-like curses features                     #
#                                                                            #
##############################################################################

echo
echo Testing for SystemV-like curses features...
cat >testing.c <<!



#include <stdio.h>
#include <ctype.h>
#include <curses.h>

main() {
	int key;
	WINDOW *selwin;

	initscr();
	raw();
	noecho();
	intrflush(stdscr,FALSE);
	keypad(stdscr,TRUE);
	idlok(stdscr,FALSE);

	clear();
	selwin = newwin(4,0,13,80);
	box(selwin,0,0);
	refresh();
	move(4,0);	addch(ACS_ULCORNER);
	move(4,79);	addch(ACS_URCORNER);
	move(17,0);	addch(ACS_LLCORNER);
	move(17,79);	addch(ACS_LRCORNER);
	move(4,1);	addch(ACS_HLINE);
	move(17,1);	addch(ACS_HLINE);
	move(1,0);	addch(ACS_VLINE);
	move(1,79);	addch(ACS_VLINE);
	refresh();

	standout();
	standend();

	key = wgetch(stdscr);
	if (key=='-')	key = KEY_UP;
	if (key=='+')	key = KEY_DOWN;
	if (key==1)	key = KEY_F(1);
	if (key==2)	key = KEY_PPAGE;
	if (key==6)	key = KEY_NPAGE;

	clearok(stdscr,TRUE);
	clearok(stdscr,FALSE);
	clear();
	refresh();
	endwin();
	exit(0);
	}


!

if cc testing.c -o testing 2>/dev/null; then
	echo ...advanced features are available.
	echo "#define ADVANCED_CURSES" >>sysdep.h
else
	echo ...not available.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



##############################################################################
#                                                                            #
#              Test to see if we have the /proc filesystem                   #
#                                                                            #
##############################################################################

echo
echo Testing for /proc filesystem...
cat >testing.c <<!

#include <stdio.h>

main(argc,argv)
int argc;
char *argv[]; {
	FILE *pr;
	char buf[256];

	sprintf(buf,"/proc/%d/cmdline",getpid());
	pr = fopen(buf,"r");
	if (pr==NULL) exit(1);
	if (fgets(buf,256,pr)==NULL) exit(2);
	fclose(pr);
	if (strcmp(buf,argv[0])) exit(3);
	exit(0);
	}

!

cc testing.c -o testing 2>/dev/null
if testing; then
	echo ...this system has it.
	echo "#define HAVE_PROC_FS" >>sysdep.h
else
	echo ...not available.
fi
rm -f testing testing.o testing.c >/dev/null 2>&1



