/*
 * This program converts Citadel/UX 3.1X data files to 3.2X format.
 *
 * Only run the program ONCE.
 * BACK UP YOUR OLD FILES before running this program.
 *
 * If you are not upgrading, you do NOT need this program.
 *
 */

#include <stdio.h>


#include "sysdep.h"
#include "sysconfig.h"
#define REV_LEVEL 324

#undef	tolower
#define tolower(x)	( ((x>='A')&&(x<='Z')) ? (x+'a'-'A') : x )
#define strucmp(lstr,rstr) struncmp(lstr,rstr,32767)
#define NEW_CONFIG

typedef unsigned char CIT_UBYTE;

struct config {
	char c_nodename[16];		/* UUCP and Citadel nodename        */
	char c_fqdn[64];		/* Fully Qualified Domain Name      */
	char c_humannode[21];		/* Long name of system              */
	char c_phonenum[16];		/* Dialup number of system          */
	int c_bbsuid;			/* UID of the bbs-only user         */
	int c_pwcrypt;			/* password encryption seed         */
	char c_creataide;		/* room creator = room aide  flag   */
	int c_sleeping;			/* watchdog timer setting           */
	char c_initax;			/* initial access level             */
	char c_regiscall;		/* call number to register on       */
	char c_twitdetect;		/* twit detect flag                 */
	char c_twitroom[20];		/* twit detect msg move to room     */
	int c_defent;			/* command generated by <E> key     */
	char c_moreprompt[80];		/* paginator prompt                 */
	char c_restrict;		/* restrict Internet mail flag      */
	long c_msgbase;			/* size of message base             */
	char c_bbs_city[32];		/* city and state you are located in*/
	char c_sysadm[26];		/* name of system administrator     */
	char c_bucket_dir[15];		/* bit bucket for files...	    */
	int c_setup_level;		/* what rev level we've setup to    */
	int c_maxsessions;		/* maximum concurrent sessions      */
	};

#define NODENAME		config.c_nodename
#define FQDN			config.c_fqdn
#define HUMANNODE		config.c_humannode
#define PHONENUM		config.c_phonenum
#define BBSUID			config.c_bbsuid
#define PWCRYPT			config.c_pwcrypt
#define CREATAIDE		config.c_creataide
#define SLEEPING		config.c_sleeping
#define INITAX			config.c_initax
#define REGISCALL		config.c_regiscall
#define TWITDETECT		config.c_twitdetect
#define TWITROOM		config.c_twitroom
#define RESTRICT_INTERNET	config.c_restrict
#define MM_FILELEN		config.c_msgbase

struct citsession {
	int cs_pid;
	char cs_name[26];
	char cs_room[20];
	long cs_lastupdt;
	};

struct usersupp {			/* User record                      */
	int USuid;			/* userid (==BBSUID for bbs only)   */
	char password[20];		/* password (for BBS-only users)    */
	long lastseen[MAXROOMS];	/* Last message seen in each room   */
	char generation[MAXROOMS];	/* Generation # (for private rooms) */
	char forget[MAXROOMS];		/* Forgotten generation number      */
	long mailnum[MAILSLOTS];	/* Message #'s of each mail message */
	long mailpos[MAILSLOTS];	/* Disk positions of each mail      */
	unsigned flags;			/* See US_ flags below              */
	int timescalled;		/* Total number of logins           */
	int posted;			/* Number of messages posted (ever) */
	char fullname[26];		/* Name for Citadel messages & mail */
	char axlevel;			/* Access level                     */
	CIT_UBYTE USscreenwidth;	/* Screen width                     */
	CIT_UBYTE USscreenheight;	/* Screen height                    */
	char spare[1];			/* spare bytes in the user account  */
	long eternal;			/* Eternal user number              */
	long lastcall;			/* Last time the user called        */
	char USname[30];
	char USaddr[25];
	char UScity[15];
	char USstate[3];
	char USzip[10];
	char USphone[11];
				};

#define US_NEEDVALID	1		/* User needs to be validated       */
#define US_PERM		4		/* Permanent user                   */
#define US_LASTOLD	16		/* Print last old message with new  */
#define US_EXPERT	32		/* Experienced user		    */
#define US_UNLISTED	64		/* Unlisted userlog entry           */
#define US_NOPROMPT	128		/* Don't prompt after each message  */
#define US_DISAPPEAR	512		/* Use "disappearing msg prompts"   */
#define US_REGIS	1024		/* Registered user                  */
#define US_PAGINATOR	2048		/* Pause after each screen of text  */
#define US_INTERNET	4096		/* UUCP/Internet mail privileges    */

/* this is a mask for all of the bits the user is allowed to change */
#define US_USER_SET	(US_LASTOLD | US_EXPERT | US_UNLISTED | \
			US_NOPROMPT | US_DISAPPEAR | US_PAGINATOR)
/****************************************************************************/

struct msgmain {
	long MMlowest;			/* lowest message number in file    */
	long MMhighest;			/* highest message number in file   */
	long MMcurpos;			/* Current position in file         */
	unsigned MMflags;		/* Global system flags              */
		};

#define MM_BUSY		1		/* Another process is using file    */
#define MM_CHAOS	2		/* Something is really messed up    */
#define MM_VALID	4		/* New users need validating        */
#define MM_USEINUSE	8		/* Use table is currently in use    */

/****************************************************************************/

struct smreturn {			/* Return from the send_message()   */
	long smnumber;			/* Message number (if sent)	    */
	long smpos;			/* Position in file (if sent)       */
	int smerror;			/* Error code                       */
		};

/****************************************************************************/

struct quickroom {
	char QRname[20];		/* Max. len is 19, plus null term   */
	char QRpasswd[10];		/* Only valid if it's a private rm  */
	long QRroomaide;		/* User number of room aide         */
	long QRhighest;			/* Highest message NUMBER in room   */
	char QRgen;			/* Generation number of room        */
	unsigned QRflags;		/* See flag values below            */
	char QRdirname[15];		/* Directory name, if applicable    */
	long QRinfo;			/* Info file update relative to msgs*/
		};

#define QR_BUSY		1		/* This would be a nice addition :-)*/
#define QR_INUSE	2		/* Set if in use, clear if avail    */
#define QR_PRIVATE	4		/* Set for any type of private room */
#define QR_PASSWORDED	8		/* Set if there's a password too    */
#define QR_GUESSNAME	16		/* Set if it's a guessname room     */
#define QR_DIRECTORY	32		/* Directory room                   */
#define QR_UPLOAD	64		/* Allowed to upload                */
#define QR_DOWNLOAD	128		/* Allowed to download              */
#define QR_VISDIR	256		/* Visible directory                */
#define QR_ANONONLY	512		/* Anonymous-Only room              */
#define QR_ANON2	1024		/* Anonymous-Option room            */
#define QR_NETWORK	2048		/* Shared network room              */
#define QR_PREFONLY	4096		/* Preferred status needed to enter */
#define QR_READONLY	8192		/* Aide status required to post     */

struct fullroom {
	long FRnum[MSGSPERRM];		/* Message NUMBERS		    */
	long FRpos[MSGSPERRM];		/* Message POSITIONS in master file */
		};

struct calllog {
	char CLfullname[30];		/* Name of user                     */
	long CLtime;			/* Date/time of record              */
	unsigned CLflags;		/* Info on record		    */
	};
#define CL_IN300	1		/* Carrier, 300 baud		    */
#define CL_IN1200	2		/* Carrier, 1200 baud		    */
#define CL_IN2400	4		/* Carrier, 2400 baud		    */
#define CL_INOTHER	8		/* Console or misc. baud rate	    */
#define CL_LOGIN	16		/* CLfullname logged in		    */
#define CL_NEWUSER	32		/* CLfullname is a new user	    */
#define CL_BADPW	64		/* Bad attempt at CLfullname's pw   */
#define CL_TERMINATE	128		/* Logout - proper termination	    */
#define CL_DROPCARR	256		/* Logout - dropped carrier	    */
#define CL_SLEEPING	512		/* Logout - sleeping		    */
#define CL_PWCHANGE	1024		/* CLfullname changed passwords     */

/* Miscellaneous                                                            */

#define MES_NORMAL	65		/* Normal message                   */
#define MES_ANON	66		/* "****" header                    */
#define MES_AN2		67		/* "Anonymous" header               */

#define M_ERROR		(-1)	/* Can't send message due to bad address   */
#define M_LOCAL		0	/* Local message, do no network processing */
#define M_UUCP		1	/* Translate to ASCII and use uucp main    */
#define M_BINARY	2	/* Process recipient and send through bin  */

/****************************************************************************/

struct recentmsg {
	char RMnodename[10];
	long RMnum;			/* Number or time of message        */
	};

#define NEWREGISTER	0		/* new user to register */
#define REREGISTER	1		/* existing user reregistering */

#define READ_HEADER	2
#define READ_MSGBODY	3


struct old_usersupp {			/* User record                      */
	int oUSuid;			/* userid (==BBSUID for bbs only)   */
	char opassword[20];		/* password (for BBS-only users)    */
	long olastseen[MAXROOMS];	/* Last message seen in each room   */
	char ogeneration[MAXROOMS];	/* Generation # (for private rooms) */
	char oforget[MAXROOMS];		/* Forgotten generation number      */
	long omailnum[MAILSLOTS];	/* Message #'s of each mail message */
	long omailpos[MAILSLOTS];	/* Disk positions of each mail      */
	unsigned oflags;		/* See US_ flags below              */
	int oscreenwidth;		/* For formatting messages          */
	int otimescalled;		/* Total number of logins           */
	int oposted;			/* Number of messages posted (ever) */
	char ofullname[26];		/* Name for Citadel messages & mail */
	char oaxlevel;			/* Access level                     */
	char ospare[3];			/* spare bytes in the user account  */
	long oeternal;			/* Eternal user number              */
	long olastcall;			/* Last time the user called        */
				};

struct old_quickroom {
	char oQRname[20];		/* Max. len is 19, plus null term   */
	char oQRpasswd[10];		/* Only valid if it's a private rm  */
	long oQRroomaide;		/* User number of room aide         */
	long oQRhighest;		/* Highest message NUMBER in room   */
	char oQRgen;			/* Generation number of room        */
	unsigned oQRflags;		/* See flag values below            */
	char oQRdirname[15];		/* Directory name, if applicable    */
		};

struct old_registration {
	long oRGeternal;
	char oRGname[30];
	char oRGaddr[25];
	char oRGcity[15];
	char oRGstate[3];
	char oRGzip[6];
	char oRGphone[11];
	};

long ftell();

extern int errno;
struct config config;

FILE *efopen(name,mode)
char *name;
char *mode; {
	FILE *fp;
	fp=fopen(name,mode);
	if (fp!=NULL) return(fp);
	fprintf(stderr,"efopen: cannot open '%s', errno=%d\n",name,errno);
	exit(errno);
	}

main() {
	int a,in,out,merged;
	char buf[64];
	long pos;
	
	FILE *from,*to;

	struct old_quickroom qrf;
	struct quickroom qrt;
	struct old_usersupp usf;
	struct usersupp ust;
	struct old_registration ref;

	printf("This program converts your data files from version 3.1X\n");
	printf("to version 3.2X.  BACK UP YOUR DATA FIRST, and then run\n");
	printf("program ONCE.  If this is a new installation, and you\n");
	printf("are not upgrading, then you do not need to run this\n");
	printf("program.\n\nDo you still wish to continue? ");
	gets(buf);

	if (tolower(buf[0])!='y') exit(0);
	
	printf("\n*** Phase 1 - convert room index (quickroom)\n");

	from = efopen("quickroom","r");
	to = efopen("quickroom.new","w");
	in = 0;
	out = 0;
	for (a=0; a<MAXROOMS; ++a) {
		in = in + 
			fread((char *)&qrf,sizeof(struct old_quickroom),1,from);
		strcpy(qrt.QRname,qrf.oQRname);
		strcpy(qrt.QRpasswd,qrf.oQRpasswd);
		qrt.QRroomaide = qrf.oQRroomaide;
		qrt.QRhighest = qrf.oQRhighest;
		qrt.QRgen = qrf.oQRgen;
		qrt.QRflags = qrf.oQRflags;
		strcpy(qrt.QRdirname,qrf.oQRdirname);
		out = out + fwrite((char *)&qrt,sizeof(struct quickroom),1,to);
		}
	fclose(from);
	fclose(to);
	printf("    Records read:    %d\n",in);
	printf("    Records written: %d\n",out);
	if (in==out) {
		system("mv quickroom quickroom.31");
		system("mv quickroom.new quickroom");
		printf("    Old format file saved as 'quickroom.31'\n");
		}

	printf("\n*** Phase 2 - convert userlog (usersupp/registration)\n");

	from = efopen("usersupp","r");
	to = efopen("usersupp.new","w");
	in = 0;
	out = 0;
	while (fread((char *)&usf,sizeof(struct old_usersupp),1,from)>0) {
		++in;
		ust.USuid = usf.oUSuid;	
		for (a=0; a<20; ++a) ust.password[a] = usf.opassword[a];
		for (a=0; a<MAXROOMS; ++a) {
			ust.lastseen[a] = usf.olastseen[a];
			ust.generation[a] = usf.ogeneration[a];
			ust.forget[a] = usf.oforget[a];
			}
		for (a=0; a<MAILSLOTS; ++a) {
			ust.mailnum[a] = usf.omailnum[a];
			ust.mailpos[a] = usf.omailpos[a];
			}
		ust.flags = usf.oflags;
		ust.timescalled = usf.otimescalled;
		ust.posted = usf.oposted;
		strcpy(ust.fullname,usf.ofullname);
		ust.axlevel = usf.oaxlevel;
		ust.eternal = usf.oeternal;
		ust.lastcall = usf.olastcall;
		out = out + fwrite((char *)&ust,sizeof(struct usersupp),1,to);
		}
	fclose(from);
	fclose(to);
	printf("    Records read:    %d\n",in);
	printf("    Records written: %d\n",out);
	if (in==out) {
		system("mv usersupp usersupp.31");
		system("mv usersupp.new usersupp");
		printf("    Old format file saved as 'usersupp.31'\n");
		}

	printf("\n*** Phase 3 - merge info file pointers\n");
	in = 0;
	out = 0;
	merged = 0;
	to=efopen("quickroom","r+");
	for (a=0; a<MAXROOMS; ++a) {
		fseek(to,(long)a*sizeof(struct quickroom),0);
		in = in + fread((char *)&qrt,sizeof (struct quickroom),1,to);

		sprintf(buf,"./info/%d.ptr",a);
		from=fopen(buf,"r");
		if (from!=NULL) {
			fscanf(from,"%ld",&qrt.QRinfo);
			++merged;
			fclose(from);
			}

		fseek(to,(long)a*sizeof(struct quickroom),0);
		out = out + fwrite((char *)&qrt,sizeof(struct quickroom),1,to);
		}
	fclose(to);
	printf("    Records read:    %d\n",in);
	printf("    Records written: %d\n",out);
	printf("    Records merged:  %d\n",merged);

	printf("\n*** Phase 4 - merge registration into userlog\n");
	in = 0;
	out = 0;
	merged = 0;
	to=efopen("usersupp","r+");
	from=efopen("registration","r");

	while (pos=ftell(to),
	      fread((char *)&ust,sizeof(struct usersupp),1,to)==1) {
		in = in + 1;

		fseek(from,0L,0);
		while (fread((char *)&ref,
		      sizeof(struct old_registration),1,from)>0) {
			if (ref.oRGeternal == ust.eternal) {
				strcpy(ust.USname,ref.oRGname);
				strcpy(ust.USaddr,ref.oRGaddr);
				strcpy(ust.UScity,ref.oRGcity);
				strcpy(ust.USstate,ref.oRGstate);
				strcpy(ust.USzip,ref.oRGzip);
				strcpy(ust.USphone,ref.oRGphone);
				++merged;
				}
			}

		fseek(to,pos,0);
		out = out + fwrite((char *)&ust,(sizeof(struct usersupp)),1,to);
		fseek(to,pos+(long)sizeof(struct usersupp),0);
		}
	fclose(to);
	fclose(from);
	printf("    Records read:    %d\n",in);
	printf("    Records written: %d\n",out);
	printf("    Records merged:  %d\n",merged);

	exit(0);
	}
