/* conv_32_40.c
 *
 * This program converts your data files from 3.2x format to 4.0x format.
 *
 * see copyright.doc for copyright information
 *
 * Instead of #including citadel.h, it is copied directly into this file,
 * to guarantee that the correct version is used.
 *
 */


#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>

/* system customizations are in sysconfig.h */
#include "sysdep.h"
#include "sysconfig.h"
#include "ipcdef.h"
#define CITADEL	"Citadel/UX 4.00"
#define REV_LEVEL 400
#define SERVER_TYPE 0

#ifdef tolower
#undef	tolower
#endif
#define tolower(x)	( ((x>='A')&&(x<='Z')) ? (x+'a'-'A') : x )
#define strucmp(lstr,rstr) struncmp(lstr,rstr,32767)
#define NEW_CONFIG

typedef unsigned char CIT_UBYTE;

struct config {
	char c_nodename[16];		/* UUCP and Citadel nodename        */
	char c_fqdn[64];		/* Fully Qualified Domain Name      */
	char c_humannode[21];		/* Long name of system              */
	char c_phonenum[16];		/* Dialup number of system          */
	int c_bbsuid;			/* UID of the bbs-only user         */
	int c_pwcrypt;			/* password encryption seed         */
	char c_creataide;		/* room creator = room aide  flag   */
	int c_sleeping;			/* watchdog timer setting           */
	char c_initax;			/* initial access level             */
	char c_regiscall;		/* call number to register on       */
	char c_twitdetect;		/* twit detect flag                 */
	char c_twitroom[20];		/* twit detect msg move to room     */
	int c_defent;			/* command generated by <E> key     */
	char c_moreprompt[80];		/* paginator prompt                 */
	char c_restrict;		/* restrict Internet mail flag      */
	long c_msgbase;			/* size of message base             */
	char c_bbs_city[32];		/* city and state you are located in*/
	char c_sysadm[26];		/* name of system administrator     */
	char c_bucket_dir[15];		/* bit bucket for files...	    */
	int c_setup_level;		/* what rev level we've setup to    */
	int c_maxsessions;		/* maximum concurrent sessions      */
	};

#define NODENAME		config.c_nodename
#define FQDN			config.c_fqdn
#define HUMANNODE		config.c_humannode
#define PHONENUM		config.c_phonenum
#define BBSUID			config.c_bbsuid
#define PWCRYPT			config.c_pwcrypt
#define CREATAIDE		config.c_creataide
#define INITAX			config.c_initax
#define REGISCALL		config.c_regiscall
#define TWITDETECT		config.c_twitdetect
#define TWITROOM		config.c_twitroom
#define RESTRICT_INTERNET	config.c_restrict
#define MM_FILELEN		config.c_msgbase

struct citsession {
	int cs_pid;
	char cs_name[26];
	char cs_room[20];
	long cs_lastupdt;
	};

struct ousersupp {			/* User record                      */
	int oUSuid;			/* userid (==BBSUID for bbs only)   */
	char opassword[20];		/* password (for BBS-only users)    */
	long olastseen[MAXROOMS];	/* Last message seen in each room   */
	char ogeneration[MAXROOMS];	/* Generation # (for private rooms) */
	char oforget[MAXROOMS];		/* Forgotten generation number      */
	long omailnum[MAILSLOTS];	/* Message #'s of each mail message */
	long omailpos[MAILSLOTS];	/* Disk positions of each mail      */
	unsigned oflags;		/* See US_ flags below              */
	int otimescalled;		/* Total number of logins           */
	int oposted;			/* Number of messages posted (ever) */
	char ofullname[26];		/* Name for Citadel messages & mail */
	char oaxlevel;			/* Access level                     */
	CIT_UBYTE oUSscreenwidth;	/* Screen width                     */
	CIT_UBYTE oUSscreenheight;	/* Screen height                    */
	char ospare[1];			/* spare bytes in the user account  */
	long oeternal;			/* Eternal user number              */
	long olastcall;			/* Last time the user called        */
	char oUSname[30];		/* Real name (i.e. not a handle)    */
	char oUSaddr[25];		/* Street address                   */
	char oUScity[15];		/* Municipality                     */
	char oUSstate[3];		/* State or province                */
	char oUSzip[10];		/* ZIP code                         */
	char oUSphone[11];		/* Voice telephone number           */
	};

struct usersupp {			/* User record                      */
	int USuid;			/* userid (==BBSUID for bbs only)   */
	char password[20];		/* password (for BBS-only users)    */
	long lastseen[MAXROOMS];	/* Last message seen in each room   */
	char generation[MAXROOMS];	/* Generation # (for private rooms) */
	char forget[MAXROOMS];		/* Forgotten generation number      */
	long mailnum[MAILSLOTS];	/* Message #'s of each mail message */
	long mailpos[MAILSLOTS];	/* Disk positions of each mail      */
	unsigned flags;			/* See US_ flags below              */
	int timescalled;		/* Total number of logins           */
	int posted;			/* Number of messages posted (ever) */
	char fullname[26];		/* Name for Citadel messages & mail */
	char axlevel;			/* Access level                     */
	CIT_UBYTE USscreenwidth;	/* Screen width                     */
	CIT_UBYTE USscreenheight;	/* Screen height                    */
	char spare[1];			/* spare bytes in the user account  */
	long eternal;			/* Eternal user number              */
	long lastcall;			/* Last time the user called        */
	char USname[30];		/* Real name (i.e. not a handle)    */
	char USaddr[25];		/* Street address                   */
	char UScity[15];		/* Municipality                     */
	char USstate[3];		/* State or province                */
	char USzip[10];			/* ZIP code                         */
	char USphone[11];		/* Voice telephone number           */
	char USemail[32]; 		/* (Not yet implemented)            */
	int logged_time; 		/* (Not yet implemented)            */
	int time_limit; 		/* (Not yet implemented)            */
	};

#define OLD_US_SIZE sizeof(struct ousersupp)

#define US_NEEDVALID	1		/* User needs to be validated       */
#define US_PERM		4		/* Permanent user                   */
#define US_LASTOLD	16		/* Print last old message with new  */
#define US_EXPERT	32		/* Experienced user		    */
#define US_UNLISTED	64		/* Unlisted userlog entry           */
#define US_NOPROMPT	128		/* Don't prompt after each message  */
#define US_DISAPPEAR	512		/* Use "disappearing msg prompts"   */
#define US_REGIS	1024		/* Registered user                  */
#define US_PAGINATOR	2048		/* Pause after each screen of text  */
#define US_INTERNET	4096		/* UUCP/Internet mail privileges    */

/* this is a mask for all of the bits the user is allowed to change */
#define US_USER_SET	(US_LASTOLD | US_EXPERT | US_UNLISTED | \
			US_NOPROMPT | US_DISAPPEAR | US_PAGINATOR)
/****************************************************************************/

struct msgmain {
	long MMlowest;			/* lowest message number in file    */
	long MMhighest;			/* highest message number in file   */
	long MMcurpos;			/* Current position in file         */
	unsigned MMflags;		/* Global system flags              */
	};

#define MM_BUSY		1		/* Another process is using file    */
#define MM_CHAOS	2		/* Something is really messed up    */
#define MM_VALID	4		/* New users need validating        */
#define MM_USEINUSE	8		/* Use table is currently in use    */

/****************************************************************************/

struct smreturn {			/* Return from the send_message()   */
	long smnumber;			/* Message number (if sent)	    */
	long smpos;			/* Position in file (if sent)       */
	int smerror;			/* Error code                       */
	};

/****************************************************************************/

struct oquickroom {
	char oQRname[20];		/* Max. len is 19, plus null term   */
	char oQRpasswd[10];		/* Only valid if it's a private rm  */
	long oQRroomaide;		/* User number of room aide         */
	long oQRhighest;		/* Highest message NUMBER in room   */
	char oQRgen;			/* Generation number of room        */
	unsigned oQRflags;		/* See flag values below            */
	char oQRdirname[15];		/* Directory name, if applicable    */
	long oQRinfo;			/* Info file update relative to msgs*/
	};

struct quickroom {
	char QRname[20];		/* Max. len is 19, plus null term   */
	char QRpasswd[10];		/* Only valid if it's a private rm  */
	long QRroomaide;		/* User number of room aide         */
	long QRhighest;			/* Highest message NUMBER in room   */
	char QRgen;			/* Generation number of room        */
	unsigned QRflags;		/* See flag values below            */
	char QRdirname[15];		/* Directory name, if applicable    */
	long QRinfo;			/* Info file update relative to msgs*/
	char QRfloor;			/* (Not yet implemented)            */
	};

#define OLD_QR_SIZE sizeof(struct oquickroom)

#define QR_INUSE	2		/* Set if in use, clear if avail    */
#define QR_PRIVATE	4		/* Set for any type of private room */
#define QR_PASSWORDED	8		/* Set if there's a password too    */
#define QR_GUESSNAME	16		/* Set if it's a guessname room     */
#define QR_DIRECTORY	32		/* Directory room                   */
#define QR_UPLOAD	64		/* Allowed to upload                */
#define QR_DOWNLOAD	128		/* Allowed to download              */
#define QR_VISDIR	256		/* Visible directory                */
#define QR_ANONONLY	512		/* Anonymous-Only room              */
#define QR_ANON2	1024		/* Anonymous-Option room            */
#define QR_NETWORK	2048		/* Shared network room              */
#define QR_PREFONLY	4096		/* Preferred status needed to enter */
#define QR_READONLY	8192		/* Aide status required to post     */

/* Private rooms are always flagged with QR_PRIVATE. If neither QR_PASSWORDED
 * or QR_GUESSNAME is set, then it is invitation-only. Passworded rooms are
 * flagged with both QR_PRIVATE and QR_PASSWORDED while guess-name rooms are
 * flagged with both QR_PRIVATE and QR_GUESSNAME. DO NOT set all three flags.
 */
/****************************************************************************/

struct fullroom {
	long FRnum[MSGSPERRM];		/* Message NUMBERS		    */
	long FRpos[MSGSPERRM];		/* Message POSITIONS in master file */
		};

void progress(curr,max)
long curr;
long max; {
	static int dots;
	int pos;
	
	if (curr==0L) {
		printf("--------------------------------------");
		printf("--------------------------------------\r");
		fflush(stdout);
		dots = 0;
		}

	pos = (curr * 72) / max;
	while (dots < pos) {
		putc('*',stdout);
		fflush(stdout);
		++dots;
		}

	if (dots == 72) {
		printf("\r                                      ");
		printf("                                      \r");
		fflush(stdout);
		}

	}






main() {
	struct config config;
	FILE *fp,*ofp;
	int a;
	struct usersupp usersupp;
	struct quickroom quickroom;
	long numrecs,count;
	


	fp=fopen("citadel.config","a");
	if (fp==NULL) {
		fprintf(stderr,"conv_32_40: cannot append citadel.config\n");
		exit(1);
		}
	for (a=0; a<sizeof(struct config); ++a) putc(0,fp);
	fclose(fp);

	/* now we re-open it, and read the old or blank configuration */
	fp=fopen("citadel.config","r");
	if (fp==NULL) {
		fprintf(stderr,"conv_32_40: cannot open citadel.config\n");
		exit(1);
		}
	fread((char *)&config,sizeof(struct config),1,fp);
	fclose(fp);

	printf("Your data files are currently at version %d.%d\n",
		(config.c_setup_level / 100),
		(config.c_setup_level % 100));


	if ((config.c_setup_level / 10) < 32) {
printf("Your data files are too old to be updated with this program.\n");
printf("If you are upgrading from version 3.1X, you can run the conv_31_32\n");
printf("program to upgrade to 3.2X, and then run this program to upgrade to\n");
printf("4.0X.  Otherwise, you will need to erase your files and install a\n");
printf("new system.\n");
		exit(1);
		}

	if ((config.c_setup_level / 10) == 40) {
printf("Already up to date.  It is not necessary to run this program.\n");
		exit(0);
		}

	if ((config.c_setup_level / 10) > 40) {
printf("This is not the correct program to upgrade your files.\n");
		}



	printf("*** Phase 1: upgrade usersupp file\n");
	system("cp usersupp usersupp.32");
	printf("old size: %d   new size: %d\n",OLD_US_SIZE,sizeof(struct usersupp));
	fp = fopen("usersupp.32","r");
	fseek(fp,0L,2);
	numrecs = (ftell(fp) / OLD_US_SIZE);
	fseek(fp,0L,0);
	ofp = fopen("usersupp","w");

	for (count=1L; count<=numrecs; ++count) {
		fread((char *)&usersupp,OLD_US_SIZE,1,fp);
		fwrite((char *)&usersupp,sizeof(struct usersupp),1,ofp);
		progress(count,numrecs);
		}
	
	fclose(fp);
	fclose(ofp);



	printf("*** Phase 2: upgrade quickroom file\n");
	system("cp quickroom quickroom.32");
	printf("old size: %d   new size: %d\n",OLD_QR_SIZE,sizeof(struct quickroom));
	fp = fopen("quickroom.32","r");
	fseek(fp,0L,2);
	numrecs = (long)MAXROOMS;
	fseek(fp,0L,0);
	ofp = fopen("quickroom","w");

	for (count=1L; count<=numrecs; ++count) {
		fread((char *)&quickroom,OLD_QR_SIZE,1,fp);
		fwrite((char *)&quickroom,sizeof(struct quickroom),1,ofp);
		progress(count,numrecs);
		}
	
	fclose(fp);
	fclose(ofp);


	printf("*** Phase 3: rewrite configuration record\n");
	config.c_setup_level = REV_LEVEL;

	fp=fopen("citadel.config","w");
	fwrite((char *)&config,sizeof(struct config),1,fp);
	fclose(fp);

	exit(0);
	}
