/*
 * ipc_c_std.c
 * 
 * Citadel/UX client/server IPC - client module using pipes
 *
 */

#include <stdio.h>
#include <signal.h>

/*
 * If server_is_local is set to nonzero, the client assumes that it is running
 * on the same computer as the server.  Several things happen when this is
 * the case, including the ability to map a specific tty to a particular login
 * session in the "<W>ho is online" listing, the ability to run external
 * programs, and the ability to download files directly off the disk without
 * having to first fetch them from the server.
 * Set the flag to 1 if this IPC is local (as is the case with pipes, or a
 * network session to the local machine) or 0 if the server is executing on
 * a remote computer.
 */
char server_is_local = 1;

int sndpipe[2];
int rcvpipe[2];
FILE *tracefp = NULL;

/*
 * input string from pipe
 */
void serv_gets(strbuf)
char strbuf[]; {
	int ch,len,rlen;
	char buf[2];

	len = 0;
	strcpy(strbuf,"");
	do {
		rlen = read(rcvpipe[0],&buf[0],1);
		ch = (int) buf[0];
		if (rlen<1) {
			ch = 0;
			fprintf(stderr,"citadel: connection to server lost\n");
			logoff(3);
			}
		strbuf[len++] = ch;
		} while((ch!=10)&&(ch!=13)&&(ch!=0)&&(len<255));
	strbuf[len-1] = 0;
	if (tracefp != NULL) fprintf(tracefp,"<%s\n",strbuf);
	}



/*
 * input binary data from pipe
 */
void serv_read(buf,bytes)
char buf[];
int bytes; {
	int len,rlen;

	len = 0;
	while(len<bytes) {
		rlen = read(rcvpipe[0],&buf[len],bytes-len);
		if (rlen<1) {
			printf("\rcitadel: connection to server lost\n");
			logoff(3);
			}
		len = len + rlen;
		}
	}



/*
 * attach to server
 */
void attach_to_server(argc,argv)
int argc;
char *argv[]; {
	int stdinhold,stdouthold;
	int servpid;

	if ((argc==3)&&(!strcmp(argv[1],"-t"))) tracefp=fopen(argv[2],"w");

	if (pipe(sndpipe)!=0) {
		fprintf(stderr,"ipc_c_std: cannot create c->s pipe\n");
		exit(1);
		}

	if (pipe(rcvpipe)!=0) {
		fprintf(stderr,"ipc_c_std: cannot create s->c pipe\n");
		exit(1);
		}

	stdinhold=dup(0);
	stdouthold=dup(1);
	close(0);
	close(1);

	if (dup(sndpipe[0])!=0) {
		fprintf(stderr,"ipc_c_std: didn't get fd 0\n");
		exit(1);
		}

	if (dup(rcvpipe[1])!=1) {
		fprintf(stderr,"ipc_c_std: didn't get fd 1\n");
		exit(1);
		}

	servpid=fork();
	if (servpid<0) {
		fprintf(stderr,"ipc_c_std: cannot fork\n");
		exit(1);
		}
	if (servpid==0) {
		signal(SIGINT,SIG_IGN);
		signal(SIGQUIT,SIG_IGN);
		execlp("citserver","citserver",NULL);
		fprintf(stderr,"ipc_c_std: cannot exec citserver\n");
		exit(1);
		}

	close(0);
	close(1);
	if (dup(stdinhold) != 0) {
		fprintf(stderr,"ipc_c_std: could not reset stdin\n");
		exit(1);
		}
	if (dup(stdouthold) != 1) {
		fprintf(stderr,"ipc_c_std: could not reset stdin\n");
		exit(1);
		}
	}


/*
 * send line to server
 */
void serv_puts(string)
char *string; {
	char buf[256];

	if (tracefp != NULL) fprintf(tracefp,">%s\n",string);
	sprintf(buf,"%s\n",string);
	if (write(sndpipe[1],buf,strlen(buf)) != strlen(buf)) {
		fprintf(stderr,"citadel: connection to server lost\n");
		logoff(3);
		}
	}


/*
 * send binary to server
 */
void serv_write(buf,bytes)
char buf[];
int bytes; {
	if (write(sndpipe[1],buf,bytes) != bytes) {
		fprintf(stderr,"citadel: connection to server lost\n");
		logoff(3);
		}
	}

