/*
 * ipc_c_std.c
 * 
 * Citadel/UX client/server IPC - client module using TCP/IP
 *
 */

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#define end_program(x)	logoff(x)

/*
 * If server_is_local is set to nonzero, the client assumes that it is running
 * on the same computer as the server.  Several things happen when this is
 * the case, including the ability to map a specific tty to a particular login
 * session in the "<W>ho is online" listing, the ability to run external
 * programs, and the ability to download files directly off the disk without
 * having to first fetch them from the server.
 * Set the flag to 1 if this IPC is local (as is the case with pipes, or a
 * network session to the local machine) or 0 if the server is executing on
 * a remote computer.
 */
char server_is_local = 0;

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

extern int errno;
extern char *sys_errlist[];

int serv_sock;

#ifndef DONT_PROTO_HTONS
u_short htons();
#endif
u_long inet_addr();


void timeout() {
	printf("\rConnection timed out.\n");
	end_program(1);
	}

int connectsock(host,service,protocol)
char *host;
char *service;
char *protocol; {
	struct hostent *phe;
	struct servent *pse;
	struct protoent *ppe;
	struct sockaddr_in sin;
	int s,type;

	bzero((char *)&sin,sizeof(sin));
	sin.sin_family = AF_INET;

	if (pse=getservbyname(service,protocol))
		sin.sin_port = pse->s_port;
	else if ((sin.sin_port = htons((u_short)atoi(service))) == 0) {
		fprintf(stderr,"can't get %s service entry\n",service);
		end_program(errno);
		}
	
	if (phe=gethostbyname(host))
		bcopy(phe->h_addr,(char *)&sin.sin_addr,phe->h_length);
	else if ((sin.sin_addr.s_addr = inet_addr(host))==INADDR_NONE) {
		fprintf(stderr,"can't get %s host entry\n",host);
		end_program(errno);
		}

	if ((ppe=getprotobyname(protocol))==0) {
		fprintf(stderr,"can't get %s protocol entry\n",protocol);
		end_program(errno);
		}

	if (!strcmp(protocol,"udp"))
		type = SOCK_DGRAM;
	else
		type = SOCK_STREAM;

	s = socket(PF_INET,type,ppe->p_proto);
	if (s<0) {
		fprintf(stderr,"can't create socket, errno=%d\n",errno);
		end_program(errno);
		}


	signal(SIGALRM,timeout);
	alarm(30);

	if (connect(s,(struct sockaddr *)&sin,sizeof(sin))<0) {
		fprintf(stderr,"can't connect to %s.%s, errno=%d\n",
			host,service,errno);
		end_program(errno);
		}

	alarm(0);
	signal(SIGALRM,SIG_IGN);

	return(s);
	}


/*
 * input string from pipe
 */
void serv_gets(strbuf)
char strbuf[]; {
	int ch,len,rlen;
	char buf[2];

	len = 0;
	strcpy(strbuf,"");
	do {
		rlen = read(serv_sock,&buf[0],1);
		ch = (int) buf[0];
		if (rlen<1) {
			ch = 0;
			printf("\rConnection closed by foreign host.\n");
			end_program(3);
			}
		strbuf[len++] = ch;
		} while((ch!=10)&&(ch!=13)&&(ch!=0)&&(len<255));
	strbuf[len-1] = 0;
	}

/*
 * input binary data from pipe
 */
void serv_read(buf,bytes)
char buf[];
int bytes; {
	int len,rlen;

	len = 0;
	while(len<bytes) {
		rlen = read(serv_sock,&buf[len],bytes-len);
		if (rlen<1) {
			printf("\rConnection closed by foreign host.\n");
			end_program(3);
			}
		len = len + rlen;
		}
	}


/*
 * attach to server
 */
void attach_to_server(argc,argv)
int argc;
char *argv[]; {
	if (argc==1) {
		server_is_local = 1;
		serv_sock = connectsock("localhost","citadel","tcp");
		}
	if ( (!strcmp(argv[1],"localhost"))
	   || (!strcmp(argv[1],"127.0.0.1")) ) server_is_local = 1;
	if (argc==2) serv_sock = connectsock(argv[1],"citadel","tcp");
	if (argc>=3) serv_sock = connectsock(argv[1],argv[2],"tcp");
	}


/*
 * send line to server
 */
void serv_puts(string)
char *string; {
	char buf[256];

	sprintf(buf,"%s\n",string);
	if (write(serv_sock,buf,strlen(buf)) != strlen(buf)) {
		printf("\rConnection closed by foreign host.\n");
		end_program(3);
		}
	}


/*
 * send binary to server
 */
void serv_write(buf,bytes)
char buf[];
int bytes; {

	if (write(serv_sock,buf,bytes) != bytes) {
		printf("\rConnection closed by foreign host.\n");
		end_program(3);
		}
	}

