#include <stdio.h>
#include <sys/stat.h>
#include "citadel.h"


#ifdef HAVE_FLOCK
#include <sys/file.h>
#endif

extern struct config config;
extern struct usersupp usersupp;
extern struct quickroom quickroom;
extern struct fullroom fullroom;
extern struct msgmain msgmain;
extern char curr_user[];
extern int curr_rm;
extern int logged_in;

static FILE *qrfp = NULL;
extern char temp[];

FILE *popen();

/*
 * is_known()  -  returns nonzero if room is in user's known room list
 */
int is_known(roombuf,roomnum,userbuf)
struct quickroom *roombuf;
int roomnum;
struct usersupp *userbuf; {

	if (roomnum==1) roombuf->QRhighest=userbuf->mailnum[MAILSLOTS-1];

	if ((roombuf->QRflags & QR_INUSE)
		&& ( (roomnum!=2) || (userbuf->axlevel>=6))
		&& (roombuf->QRgen != (userbuf->forget[roomnum]) )

		&& (	((roombuf->QRflags&QR_PREFONLY)==0)
		||	((userbuf->axlevel)>=5)
		)

		&& (	((roombuf->QRflags&QR_PRIVATE)==0)
   		||	((userbuf->axlevel)>=6)
   		||	(roombuf->QRgen==(userbuf->generation[roomnum]))
   		)

		) return(1);
	else return(0);
	}


/*
 * has_newmsgs()  -  returns nonzero if room has new messages
 */
int has_newmsgs(roombuf,roomnum,userbuf)
struct quickroom *roombuf;
int roomnum;
struct usersupp *userbuf; {
	if (roombuf->QRhighest > (userbuf->lastseen[roomnum]) )
		return(1);
	else return(0);
	}

/*
 * is_zapped()  -  returns nonzero if room is on forgotten rooms list
 */
int is_zapped(roombuf,roomnum,userbuf)
struct quickroom *roombuf;
int roomnum;
struct usersupp *userbuf; {
	if (roomnum==1) roombuf->QRhighest=userbuf->mailnum[MAILSLOTS-1];
	if ((roombuf->QRflags & QR_INUSE)
		&& (roombuf->QRgen == (userbuf->forget[roomnum]) )
		&& ( (roomnum!=2) || ((userbuf->axlevel)>=6))
		&& (	((roombuf->QRflags&QR_PRIVATE)==0)
   		||	((userbuf->axlevel)>=6)
   		||	(roombuf->QRgen==(userbuf->generation[roomnum]))
   		)
		) return(1);
	else return(0);
	}

/*
 * getroom()  -  retrieve room data from disk
 */
void getroom(qrbuf,room_num)
struct quickroom *qrbuf;
int room_num; {
	long pos = 0L;

	if ((qrfp != NULL) && (room_num<0)) {
		fclose(qrfp);
		return;
		}

	if (qrfp == NULL) qrfp = fopen("quickroom","r+");

	pos = (long)room_num * (long)sizeof(struct quickroom);
	if (ftell(qrfp)!=pos) fseek(qrfp,pos,0);
	fread((char *)qrbuf,sizeof(struct quickroom),1,qrfp);
	}

/*
 * lgetroom()  -  same as getroom() but locks the record (if supported)
 */
void lgetroom(qrbuf,room_num)
struct quickroom *qrbuf;
int room_num; {
	int a;

	lock_file(fileno(qrfp),
		((long)room_num * (long)sizeof(struct quickroom)),
		((long)sizeof(struct quickroom)));
		
	getroom(qrbuf,room_num);
	}


/*
 * putroom()  -  store room data on disk
 */
void putroom(qrbuf,room_num)
struct quickroom *qrbuf;
int room_num; {
	long pos = 0L;

	if ((qrfp != NULL) && (room_num<0)) {
		fclose(qrfp);
		return;
		}

	if (qrfp == NULL) qrfp = fopen("quickroom","r+");

	pos = (long)room_num * (long)sizeof(struct quickroom);
	if (ftell(qrfp)!=pos) fseek(qrfp,pos,0);
	fwrite((char *)qrbuf,sizeof(struct quickroom),1,qrfp);
	}


/*
 * lputroom()  -  same as putroom() but unlocks the record (if supported)
 */
void lputroom(qrbuf,room_num)
struct quickroom *qrbuf;
int room_num; {

	putroom(qrbuf,room_num);
	unlock_file(fileno(qrfp),
		((long)room_num * (long)sizeof(struct quickroom)),
		((long)sizeof(struct quickroom)));

	}


readmail() {
	int a;
	for (a=0; a<MSGSPERRM; ++a) {
		fullroom.FRnum[a]=0L;
		fullroom.FRpos[a]=0L;
		}
	for (a=0; a<MAILSLOTS; ++a) {
		fullroom.FRnum[a+(MSGSPERRM-MAILSLOTS)]=usersupp.mailnum[a];
		fullroom.FRpos[a+(MSGSPERRM-MAILSLOTS)]=usersupp.mailpos[a];
		}
	quickroom.QRhighest = usersupp.mailnum[MAILSLOTS-1];
	return(0);
	}


writemail() {
	int a;
	lgetuser(&usersupp,curr_user);
	for (a=0; a<MAILSLOTS; ++a) {
		usersupp.mailnum[a]=fullroom.FRnum[a+(MSGSPERRM-MAILSLOTS)];
		usersupp.mailpos[a]=fullroom.FRpos[a+(MSGSPERRM-MAILSLOTS)];
		}
	lputuser(&usersupp,curr_user);
	return(0);
	}




/*
 * get_fullroom()  -  retrieve room message pointers
 */
void get_fullroom(frbuf,room_num)
struct fullroom *frbuf;
int room_num; {
	FILE *fp;
	char filename[32];

	if (room_num != 1) {
		sprintf(filename,"./rooms/fullrm%d",room_num);
		fp = fopen(filename,"r");
		fread((char *)frbuf,sizeof(struct fullroom),1,fp);
		fclose(fp);
		}
	else {
		readmail();
		}
	}


/*
 * put_fullroom()  -  retrieve room message pointers
 */
void put_fullroom(frbuf,room_num)
struct fullroom *frbuf;
int room_num; {
	FILE *fp;
	char filename[32];

	if (room_num != 1) {
		sprintf(filename,"./rooms/fullrm%d",room_num);
		fp = fopen(filename,"r+");
		fwrite((char *)frbuf,sizeof(struct fullroom),1,fp);
		fclose(fp);
		}
	else {
		writemail();	
		}
	}



/* 
 * cmd_lrms()   -  List all accessible rooms, known or forgotten
 */
void cmd_lrms() {
	int a;
	struct quickroom qrbuf;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (getuser(&usersupp,curr_user)) {
		printf("%d Can't locate user!\n",ERROR+INTERNAL_ERROR);
		return;
		}

	printf("%d Accessible rooms:\n",LISTING_FOLLOWS);
	
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if (	(is_known(&qrbuf,a,&usersupp))
		   ||	(is_zapped(&qrbuf,a,&usersupp)) )
			printf("%s|%d\n",qrbuf.QRname,qrbuf.QRflags);
		}
	printf("000\n");
	}

/* 
 * cmd_lkra()   -  List all known rooms
 */
void cmd_lkra() {
	int a;
	struct quickroom qrbuf;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (getuser(&usersupp,curr_user)) {
		printf("%d Can't locate user!\n",ERROR+INTERNAL_ERROR);
		return;
		}

	printf("%d Known rooms:\n",LISTING_FOLLOWS);
	
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if (is_known(&qrbuf,a,&usersupp))
			printf("%s|%d\n",qrbuf.QRname,qrbuf.QRflags);
		}
	printf("000\n");
	}

/* 
 * cmd_lkrn()   -  List Known Rooms with New messages
 */
void cmd_lkrn() {
	int a;
	struct quickroom qrbuf;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (getuser(&usersupp,curr_user)) {
		printf("%d can't locate user\n",ERROR+INTERNAL_ERROR);
		return;
		}

	printf("%d list of rms w/ new msgs\n",LISTING_FOLLOWS);
	
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if (	(is_known(&qrbuf,a,&usersupp))
		   &&	(has_newmsgs(&qrbuf,a,&usersupp)) )
			printf("%s|%d\n",qrbuf.QRname,qrbuf.QRflags);
		}
	printf("000\n");
	}

/* 
 * cmd_lkro()   -  List Known Rooms with Old (no new) messages
 */
void cmd_lkro() {
	int a;
	struct quickroom qrbuf;

	if (!logged_in) {
		printf("%d not logged in\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (getuser(&usersupp,curr_user)) {
		printf("%d can't locate user\n",ERROR+INTERNAL_ERROR);
		return;
		}

	printf("%d list of rms w/o new msgs\n",LISTING_FOLLOWS);
	
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if (	(is_known(&qrbuf,a,&usersupp))
		   &&	(!has_newmsgs(&qrbuf,a,&usersupp)) ) {
			if (!strcmp(qrbuf.QRname,"000")) printf(">");
			printf("%s|%d\n",qrbuf.QRname,qrbuf.QRflags);
			}
		}
	printf("000\n");
	}

/* 
 * cmd_lzrm()   -  List Zapped RooMs
 */
void cmd_lzrm() {
	int a;
	struct quickroom qrbuf;

	if (!logged_in) {
		printf("%d not logged in\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (getuser(&usersupp,curr_user)) {
		printf("%d can't locate user\n",ERROR+INTERNAL_ERROR);
		return;
		}

	printf("%d list of forgotten rms\n",LISTING_FOLLOWS);
	
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if (is_zapped(&qrbuf,a,&usersupp)) {
			if (!strcmp(qrbuf.QRname,"000")) printf(">");
			printf("%s|%d\n",qrbuf.QRname,qrbuf.QRflags);
			}
		}
	printf("000\n");
	}



void usergoto(where,display_result)
int where;
int display_result; {
	int a,b,c;
	int info = 0;

	curr_rm=where;
	getroom(&quickroom,curr_rm);
	lgetuser(&usersupp,curr_user);
	usersupp.forget[curr_rm]=(-1);
	usersupp.generation[curr_rm]=quickroom.QRgen;
	lputuser(&usersupp,curr_user);

	/* set info to 1 if the user needs to read the room's info file */
	if (quickroom.QRinfo > usersupp.lastseen[curr_rm]) info = 1;

	b=0; c=0;
	get_mm();
	get_fullroom(&fullroom,curr_rm);
	for (a=0; a<MSGSPERRM; ++a) {
		if (fullroom.FRnum[a]>0L)
		   if (fullroom.FRnum[a]>=msgmain.MMlowest) {
			++b;
			if (fullroom.FRnum[a]>usersupp.lastseen[curr_rm]) ++c;
			}
		}
	if (display_result) printf("%d %s|%d|%d|%d|%d|%d|%d|%d|%d\n",
		OK,quickroom.QRname,c,b,info,quickroom.QRflags,
		quickroom.QRhighest,usersupp.lastseen[curr_rm],
		((curr_rm == 1) ? 1 : 0),
		(((quickroom.QRroomaide == usersupp.eternal)
			||(usersupp.axlevel>=6)) ? 1 : 0)
		);
	set_wtmpsupp(quickroom.QRname);
	}


/* 
 * cmd_goto()  -  goto a new room
 */
void cmd_goto(gargs)
char *gargs; {
	struct quickroom QRscratch;
	int a,b,c,d;
	int ok;
	char bbb[20],towhere[32],password[20];

	if (!logged_in) {
		printf("%d not logged in\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	extract(towhere,gargs,0);
	extract(password,gargs,1);

	c=0;
	getuser(&usersupp,curr_user);
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&QRscratch,a);
		if ((a==0)&&(!strucmp(towhere,"_BASEROOM_"))) {
			strcpy(towhere,QRscratch.QRname);
			}
		strcpy(bbb,QRscratch.QRname);
		ok = 0;
		if ( 
			(strucmp(bbb,towhere)==0)
			&&	((QRscratch.QRflags&QR_INUSE)!=0)

			&& (	((QRscratch.QRflags&QR_PREFONLY)==0)
			||	(usersupp.axlevel>=5)
			)

			&& (	(a!=2) || (usersupp.axlevel>=6) )

			&& (	((QRscratch.QRflags&QR_PRIVATE)==0)
   			|| (QRscratch.QRflags&QR_GUESSNAME)
			|| (usersupp.axlevel>=6)
   			|| (QRscratch.QRflags&QR_PASSWORDED)
   			||	(QRscratch.QRgen==usersupp.generation[a])
   			)

			) ok = 1;


		if (ok==1) {

			if (  (QRscratch.QRflags&QR_PASSWORDED) &&
				(usersupp.axlevel<6) &&
				(QRscratch.QRgen!=usersupp.generation[a]) &&
				(struncmp(QRscratch.QRpasswd,password))
				) {
					printf("%d wrong or missing passwd\n",
						ERROR+PASSWORD_REQUIRED);
					return;
					}

			usergoto(a,1);
			return;
			}

		}
	printf("%d room not found\n",ERROR+ROOM_NOT_FOUND);
	}


void cmd_whok() {
	struct usersupp temp;
	FILE *fp;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}
	getuser(&usersupp,curr_user);
	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	printf("%d Who knows room:\n",LISTING_FOLLOWS);
	fp=fopen("usersupp","r");
	while(fread((char *)&temp,sizeof(struct usersupp),1,fp)>0)
		if ((quickroom.QRflags & QR_INUSE)
			&& ( (curr_rm!=2) || (temp.axlevel>=6) )
			&& (quickroom.QRgen != (temp.forget[curr_rm]) )

			&& (	((quickroom.QRflags&QR_PREFONLY)==0)
			||	(temp.axlevel>=5)
			)

			&& (	((quickroom.QRflags&QR_PRIVATE)==0)
   			||	(temp.axlevel>=6)
   			||	(quickroom.QRgen==(temp.generation[curr_rm]))
   			)

			&& (strncmp(temp.fullname,"000",3))

		) printf("%s\n",temp.fullname);
	fclose(fp);
	printf("000\n");
	}


/*
 * RDIR command for room directory
 */
void cmd_rdir() {
	char buf[256];
	char flnm[256];
	char comment[256];
	FILE *ls,*fd;
	struct stat statbuf;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	getroom(&quickroom,curr_rm);
	getuser(&usersupp,curr_user);

	if ((quickroom.QRflags & QR_DIRECTORY) == 0) {
		printf("%d not here.\n",ERROR+NOT_HERE);
		return;
		}

	if (((quickroom.QRflags & QR_VISDIR) == 0)
	   && (usersupp.axlevel<6)
	   && (usersupp.eternal != quickroom.QRroomaide)) {
		printf("%d not here.\n",ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	printf("%d %s|%s/files/%s\n",
		LISTING_FOLLOWS,config.c_fqdn,BBSDIR,quickroom.QRdirname);

	sprintf(buf,"cd %s/files/%s; ls >%s 2>/dev/null",
		BBSDIR,quickroom.QRdirname,temp);
	system(buf);

	sprintf(buf,"%s/files/%s/filedir",BBSDIR,quickroom.QRdirname);
	fd = fopen(buf,"r");
	if (fd==NULL) fd=fopen("/dev/null","r");

	ls = fopen(temp,"r");
	while (fgets(flnm,256,ls)!=NULL) {
		flnm[strlen(flnm)-1]=0;
		if (strucmp(flnm,"filedir")) {
			sprintf(buf,"%s/files/%s/%s",
				BBSDIR,quickroom.QRdirname,flnm);
			stat(buf,&statbuf);
			strcpy(comment,"");
			fseek(fd,0L,0);
			while ((fgets(buf,256,fd)!=NULL)
			    &&(strlen(comment)==0)) {
				buf[strlen(buf)-1] = 0;
				if ((!struncmp(buf,flnm,strlen(flnm)))
				   && (buf[strlen(flnm)]==' ')) 
					strcpy(comment,&buf[strlen(flnm)+1]);
				}
			printf("%s|%ld|%s\n",flnm,statbuf.st_size,comment);
			}
		}
	fclose(ls);
	fclose(fd);
	unlink(temp);

	printf("000\n");
	}

/*
 * get room parameters (aide or room aide command)
 */
void cmd_getr() {
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (curr_rm < 3) {
		printf("%d Can't edit this room.\n",ERROR+NOT_HERE);
		return;
		}

	getroom(&quickroom,curr_rm);
	printf("%d|%s|%s|%s|%d\n",
		OK,
		quickroom.QRname,
		((quickroom.QRflags & QR_PASSWORDED) ? quickroom.QRpasswd : ""),
		((quickroom.QRflags & QR_DIRECTORY) ? quickroom.QRdirname : ""),
		quickroom.QRflags);
	}


/*
 * set room parameters (aide or room aide command)
 */
void cmd_setr(args) {
	char buf[256];

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (curr_rm < 3) {
		printf("%d Can't edit this room.\n",ERROR+NOT_HERE);
		return;
		}

	lgetroom(&quickroom,curr_rm);
		extract(quickroom.QRname,args,0);
		extract(quickroom.QRpasswd,args,1);
		extract(quickroom.QRdirname,args,2);
		quickroom.QRflags = extract_int(args,3);
		if (extract_int(args,4)) {
			++quickroom.QRgen;
			if (quickroom.QRgen==100) quickroom.QRgen=1;
			}
	lputroom(&quickroom,curr_rm);
	printf("%d Ok\n",OK);
	sprintf(buf,"mkdir ./files/%s </dev/null >/dev/null 2>/dev/null",
		quickroom.QRdirname);
	system(buf);

	sprintf(buf,"%s> edited by %s",quickroom.QRname,curr_user);
	aide_message(buf);
	}



/* 
 * get the name of the room aide for this room
 */
void cmd_geta() {
	struct usersupp usbuf;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (curr_rm < 0) {
		printf("%d No current room.\n",ERROR);
		return;
		}

	if (getuserbynumber(&usbuf,quickroom.QRroomaide)==0) {
		printf("%d %s\n",OK,usbuf.fullname);
		}
	else {
		printf("%d \n",OK);
		}
	}


/* 
 * set the room aide for this room
 */
void cmd_seta(new_ra)
char *new_ra; {
	struct usersupp usbuf;
	long newu;
	
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (curr_rm < 3) {
		printf("%d Can't edit this room.\n",ERROR+NOT_HERE);
		return;
		}

	if (getuser(&usbuf,new_ra)!=0) {
		newu = (-1L);
		}
	else {
		newu = usbuf.eternal;
		}

	lgetroom(&quickroom,curr_rm);
	quickroom.QRroomaide = newu;
	lputroom(&quickroom,curr_rm);
	printf("%d Ok\n",OK);
	}


/* 
 * retrieve info file for this room
 */
void cmd_rinf() {
	char filename[64];
	FILE *info_fp;
	
	sprintf(filename,"./info/%d",curr_rm);
	info_fp = fopen(filename,"r");

	if (info_fp==NULL) {
		printf("%d No info file.\n",ERROR);
		return;
		}

	printf("%d Info:\n",LISTING_FOLLOWS);	
	fmout(80,info_fp,1);
	printf("000\n");
	fclose(info_fp);
	}

/*
 * aide command: kill the current room
 */
void cmd_kill(argbuf)
char argbuf[]; {
	char aaa[100];
	int kill_ok;
	
	kill_ok = extract_int(argbuf,0);

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (curr_rm < 3) {
		printf("%d Can't kill this room.\n",ERROR+NOT_HERE);
		return;
		}

	if (kill_ok) {
		lgetroom(&quickroom,curr_rm);
		quickroom.QRflags=0;
		lputroom(&quickroom,curr_rm);
		sprintf(aaa,"%s> killed by %s",quickroom.QRname,curr_user);
		aide_message(aaa);
		curr_rm=(-1);
		printf("%d '%s' deleted.\n",OK,quickroom.QRname);
		}
	else {
		printf("%d ok to delete.\n",OK);
		}
	}


/*
 * Find a free slot to create a new room in, or return -1 for error.
 * search_dir is the direction to search in.  1 causes this function to
 * return the first available slot, -1 gets the last available slot.
 */
int get_free_room_slot(search_dir)
int search_dir; {
	int a,st;
	struct quickroom qrbuf;

	st = ((search_dir>0) ? 3 : (MAXROOMS-1));

	for (a=st; ((a<MAXROOMS)&&(a>=3)); a=a+search_dir) {
		getroom(&qrbuf,a);
		if ((qrbuf.QRflags & QR_INUSE)==0) return(a);
		}
	return(-1);
	}


/*
 * internal code to create a new room (returns room flags)
 */
unsigned create_room(free_slot,new_room_name,new_room_type,new_room_pass)
int free_slot;
char *new_room_name;
int new_room_type;
char *new_room_pass; {
	struct quickroom qrbuf;
	struct fullroom frbuf;
	int a;

	lgetroom(&qrbuf,free_slot);
	strcpy(qrbuf.QRname,new_room_name);
	strcpy(qrbuf.QRpasswd,new_room_pass);
	qrbuf.QRflags = QR_INUSE;
	if (new_room_type > 0) qrbuf.QRflags=(qrbuf.QRflags|QR_PRIVATE);
	if (new_room_type == 1) qrbuf.QRflags=(qrbuf.QRflags|QR_GUESSNAME);
	if (new_room_type == 2) qrbuf.QRflags=(qrbuf.QRflags|QR_PASSWORDED);
	qrbuf.QRroomaide = (-1L);
	if ((new_room_type > 0)&&(CREATAIDE==1))
		qrbuf.QRroomaide=usersupp.eternal;
	qrbuf.QRhighest = 0L;
	++qrbuf.QRgen; if (qrbuf.QRgen>=126) qrbuf.QRgen=10;

	/* Initialize a blank fullroom structure */
	get_fullroom(&frbuf,free_slot);
	for (a=0; a<MSGSPERRM; ++a) {
		frbuf.FRnum[a]=0L;
		frbuf.FRpos[a]=0L;
		}
	put_fullroom(&frbuf,free_slot);

	/* save what we just did... */
	lputroom(&qrbuf,free_slot);
	}


/*
 * create a new room
 */
void cmd_cre8(args)
char args[]; {
	int cre8_ok;
	int free_slot;
	int a;
	char new_room_name[256];
	int new_room_type;
	char new_room_pass[256];
	char aaa[256];
	unsigned newflags;
	struct quickroom qrbuf;

	cre8_ok = extract_int(args,0);
	extract(new_room_name,args,1);
	new_room_name[19] = 0;
	new_room_type = extract_int(args,2);
	extract(new_room_pass,args,3);
	new_room_pass[9] = 0;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (usersupp.axlevel<3) {
		printf("%d You need higher access to create rooms.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	free_slot = get_free_room_slot(1);
	if (free_slot<0) {
		printf("%d There is no space available for a new room.\n",
			ERROR);
		return;
		}

	if (cre8_ok==0) {
		printf("%d ok to create...\n",OK);
		return;
		}

	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qrbuf,a);
		if ( (!strucmp(qrbuf.QRname,new_room_name))
		   && (qrbuf.QRflags & QR_INUSE) ) {
			printf("%d '%s' already exists.\n",
				ERROR,qrbuf.QRname);
			return;
			}
		}

	if ((new_room_type < 0) || (new_room_type > 3)) {
		printf("%d Invalid room type.\n",ERROR);
		return;
		}

	newflags = create_room(free_slot,new_room_name,
				new_room_type,new_room_pass);

	/* post a message in Aide> describing the new room */
	strcpy(aaa,new_room_name); strcat(aaa,"> created by ");
	strcat(aaa,usersupp.fullname);
	if (newflags&QR_PRIVATE) strcat(aaa," [private]");
	if (newflags&QR_GUESSNAME) strcat(aaa,"[guessname] ");
	if (newflags&QR_PASSWORDED) {
		strcat(aaa,"\n Password: ");
		strcat(aaa,new_room_pass);
		}
	aide_message(aaa); 

	sprintf(aaa,"./info/%d",free_slot);	/* delete old info file */
	unlink(aaa);	

	printf("%d '%s' has been created.\n",OK,qrbuf.QRname);
	}



void cmd_einf(ok)
char *ok; {	/* enter info file for current room */
	FILE *fp;
	char infofilename[32];
	char buf[256];

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!is_room_aide()) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (atoi(ok)==0) {
		printf("%d Ok.\n",OK);
		return;
		}

	printf("%d Send info...\n",SEND_LISTING);
	fflush(stdout);

	sprintf(infofilename,"./info/%d",curr_rm);

	fp=fopen(infofilename,"w");
	do {
		gets(buf);
		if (strcmp(buf,"000")) fprintf(fp,"%s\n",buf);
		} while(strcmp(buf,"000"));
	fclose(fp);

	/* now update the room index so people will see our new info */
	lgetroom(&quickroom,curr_rm);	/* lock so no one steps on us */
	quickroom.QRinfo = quickroom.QRhighest + 1L;
	lputroom(&quickroom,curr_rm);
	}
