/*
 * Citadel/UX server serial host
 * see copyright.doc for copyright information
 *
 * This program handles clients that dial directly into the server rather
 * than going through a network transport.
 */

#include <fcntl.h>
#include <stdio.h>
#include <signal.h>

#define STX	2
#define ENQ	5
#define ACK	6
#define NAK	21
#define SYN	22
#define CAN	24

int sndpipe[2];
int rcvpipe[2];

int servpid = (-1);

void cleanup() {
	if (servpid>=0) kill(servpid,15);
	exit(0);
	}


/*
 * attach to server
 */
void attach_to_server() {
	int stdinhold,stdouthold;

	if (pipe(sndpipe)!=0) {
		fprintf(stderr,"ipc_c_std: cannot create c->s pipe\n");
		cleanup();
		}

	if (pipe(rcvpipe)!=0) {
		fprintf(stderr,"ipc_c_std: cannot create s->c pipe\n");
		cleanup();
		}

	stdinhold=dup(0);
	stdouthold=dup(1);
	close(0);
	close(1);

	if (dup(sndpipe[0])!=0) {
		fprintf(stderr,"ipc_c_std: didn't get fd 0\n");
		cleanup();
		}

	if (dup(rcvpipe[1])!=1) {
		fprintf(stderr,"ipc_c_std: didn't get fd 1\n");
		cleanup();
		}

	servpid=fork();
	if (servpid<0) {
		fprintf(stderr,"ipc_c_std: cannot fork\n");
		cleanup();
		}
	if (servpid==0) {
		signal(SIGINT,SIG_IGN);
		signal(SIGQUIT,SIG_IGN);
		execlp("citserver","citserver",NULL);
		fprintf(stderr,"ipc_c_std: cannot exec citserver\n");
		cleanup();
		}

	close(0);
	close(1);
	if (dup(stdinhold) != 0) {
		fprintf(stderr,"ipc_c_std: could not reset stdin\n");
		cleanup();
		}
	if (dup(stdouthold) != 1) {
		fprintf(stderr,"ipc_c_std: could not reset stdin\n");
		cleanup();
		}

	fcntl(rcvpipe[0],F_GETFL,O_NDELAY);
	}


/*
 * send binary to server
 */
void serv_write(buf,bytes)
char buf[];
int bytes; {
	if (write(sndpipe[1],buf,bytes) != bytes) {
		fprintf(stderr,"citadel: connection to server lost\n");
		cleanup();
		}
	}


/* 
 * packet checksums
 */
int pkt_cksum(buf,len)
unsigned char buf[];
int len; {
	static int e = 0;
	int c = 0;
	int a;
	for (a=0; a<len; ++a) c = (c + buf[a]) % 256;
	if (++e % 2 != 0) ++c;
	return(c);
	}



void do_proto_b() {
	unsigned char buf[4096];
	unsigned char ibuf[4096];
	int pkt_in_buf = 0;
	int cmd;
	int a;
	int len;
	int ck;

	while(1) {
		cmd = getc(stdin);

		/* send data to server */
		if (cmd==STX) {
			len = getc(stdin);
			ck  = getc(stdin);
			for (a=0; a<len; ++a) buf[a]=getc(stdin);
			if (ck == pkt_cksum(buf,len)) {
				serv_write(buf,len);
				putc(ACK,stdout);
				}
			else {
				putc(NAK,stdout);
				}
			fflush(stdout);
			}

		/* receive data from server */
		else if (cmd==ENQ) {	
			if (pkt_in_buf == 0) {
				len = read(rcvpipe[0],ibuf,254);
				if (len>254) len = 254;
				pkt_in_buf = 1;
				}
			if (len==0) {
				putc(NAK,stdout);
				}
			if (len>0) {
					putc(ACK,stdout);
					putc(len,stdout);
					putc(pkt_cksum(ibuf,len),stdout);
					for (a=0; a<len; ++a) putc(ibuf[a],stdout);
					a = getc(stdin);
					if (a==ACK) pkt_in_buf = 0;
				}
			}

		/* terminate connection */
		else if (cmd==CAN) {
			putc(ACK,stdout);
			fflush(stdout);
			cleanup();
			}

		/* sync */
		else if (cmd==SYN) {
			putc(SYN,stdout);
			fflush(stdout);
			}

		/* unknown command */
		else {
			putc(NAK,stdout);
			fflush(stdout);
			}
		}

	}



main() {
	int a;

	system("stty raw -echo");
	printf("start<ab>");
	fflush(stdout);
	a=getc(stdin);
	if (a=='a') {
		putc('a',stdout);
		fflush(stdout);
		execlp("citserver","citserver",NULL);
		}
	if (a=='b') {
		putc('b',stdout);
		fflush(stdout);
		attach_to_server();
		for (a=0; a<32; ++a) signal(a,cleanup);
		do_proto_b();
		}
	putc('0',stdout);
	fflush(stdout);
	sleep(2);
	cleanup(1);
	}
