/*
 * Citadel/UX setup program
 * v2.4 / by Art Cancro
 * see copyright.doc for copyright information
 *
 * *** YOU MUST EDIT sysconfig.h >BEFORE< COMPILING SETUP ***
 */

#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>

#include "citadel.h"

#ifdef VISUAL_SETUP
#include "citcurses.h"
#endif

#include "axdefs.h"

char *setup_titles[] = {
	"",
	"Citadel and UUCP node name",
	"Fully Qualified Domain Name (FQDN)",
	"Human-readable node name",
	"Phone number",
	"BBS City and State",
	"System Administrator",
	"BBS User ID",
	"Password encryption seed",
	"'Room Creator = Room Aide' flag",
	"Server timeout period",
	"Initial access level",
	"Registration requirements",
	"Twit Detect!",
	"Twit Detect target room",
	"Maximum concurrent sessions",
	"Paginator prompt",
	"Restrict Internet mail flag",
	"Message base size",
	"Name of bit bucket subdirectory",
	};


char *setup_text[] = {

"1",
"This is the name your system is known by on a Citadel/UX network.  It",
"should be 8 characters or less, and you generally should set it to the",
"same as your system's UUCP node name.  In fact, the rules that apply when",
"choosing a UUCP node name are the same rules that apply when choosing a",
"Citadel node name - which shouldn't be a problem if they're one and the",
"same.",

"2",
"This is the name your system is known by on the Internet.",
"If you're not on the Internet, simply set this to your",
"node name followed by '.UUCP'.",

"3",
"This is a longer description of your system, readable by",
"us mere humans.  It can be up to 20 characters long and it",
"can have spaces in it.  Note that if you are part of a",
"Cit86Net, this is the name your system will be known by on",
"that network.",

"4",
"This is the main dialup number for your system.  If yours",
"can not be dialed into, then make one up!  It should be in",
"the format 'US 000 000 0000' - the US is your country code",
"(look it up if you're not in the United States) and the",
"rest is, of course, your area code and phone number.",
"This doesn't have any use in Citadel/UX, but gateways to",
"other networks may require it, and someday we may use this",
"to have the networker automatically build a BBS list.",

"5",
"Enter the city and state your system is located in.",

"6",
"Enter the name of the system administrator (which is probably you).",

"7",
"You should create a user called 'bbs', 'guest', 'citadel', or something",
"similar, that will allow users a way into your BBS.  Citadel doesn't require",
"that you put all of your users into /etc/passwd, although it will create",
"accounts automatically for those users and link them to their /etc/passwd",
"accounts, automatically logging them into their Citadel account when they",
"run Citadel.  For everyone else, however, you can simply create one public",
"login in /etc/passwd, and Citadel will maintain its own userlog for those",
"people.  For this to work, you must set the BBS User ID parameter to the",
"UID of that account.",

"8",
"Citadel uses a (very) simple password encryption scheme",
"to thwart breakins that could occur if someone snatched",
"a copy of your userlog.  This parameter is part of the",
"algorithm, so that the code can be different on each",
"system.  Once it has been set, DO NOT change it --",
"otherwise no one will be able to log in!",

"9",
"This is a boolean value.  If you set it to 1, anyone who",
"creates a class 3 (passworded) or class 4 (invitation",
"only) room will automatically become the Room Aide for",
"that room, allowing them to edit it, delete/move messages,",
"etc.  This is an administrative decision: it works well on",
"some systems, and not so well on others.  Set this to 0 to",
"disable this function.",

"10",
"This setting specifies how long a server session may sit idle before it is",
"automatically terminated.  The recommended value is 900 seconds (15 minutes).",
"Note that this has *nothing* to do with any watchdog timer that is presented",
"to the user.  The server's timeout is intended to kill idle or zombie",
"sessions running on a network, etc.  To disable, set it to 0.",

"11",
"This is the access level new users are assigned.",
"",
"The most common settings for this will be either 1, for",
"systems which require new user validation by the system",
"administrator ('sysop' is a word for people who run DOS",
"boards!), or 4, for systems which give instant access.",
"The current access levels available are:",

"12",
"'Registration' refers to the boring part of logging into a BBS for the first",
"time: typing your name, address, and telephone number.  Set this value to 1",
"to automatically do registration for new users, or 0 to not auto-register.",
"Optionally, you could set it to, say, 2, to auto-register on a user's second",
"call, but there really isn't much point to doing this.  The recommended",
"value is 1 if you've set your inital access level to 1, or 0 if you've set",
"your initial access level to something higher.",

"13",
"Every BBS has its share of problem users.  This is one",
"good way to deal with them: if you enable this option,",
"anyone you flag as a 'problem user' (access level 2) can",
"post anywhere they want, but their messages will all be",
"automatically moved to a room of your choosing.  Set this",
"value to 1 to enable Twit Detect, or 0 to disable it.",

"14",
"This is the name of the room that problem user messages",
"get moved to if you have Twit Detect enabled.",
"(Note: don't forget to *create* this room!)",

"15",
"This is the maximum number of concurrent Citadel sessions which may be",
"running at any given time.  Use this to keep very busy systems from being",
"overloaded.",
"  Set this value to 0 to allow an unlimited number of sessions.",

"16",
"This is the prompt that appears after each screenful of",
"text - for users that have chosen that option.  Usually",
"a simple '<more>' will do, but some folks like to be",
"creative...",

"17",
"If you have a gateway set up to allow Citadel users to",
"send Internet mail, with UUCP, sendmail, or whatever, and",
"you wish to restrict this to only users to whom you have",
"given this privilege, set this flag to 1.  Otherwise, set",
"it to 0 to allow everyone to send Internet mail.",
"(Obviously, if your system doesn't have the ability to",
"send mail to the outside world, this is all irrelevant.)",

"18",
"This is the size of the file ALL messages are stored in.",
"The file is a fixed size, is created during setup, and",
"never grows or shrinks.  Once you have created the file,",
"you cannot change its size.",
"Two megabytes (2097152 bytes) is fine for most systems.",

"19",
"Select the name of a subdirectory (relative to the main",
"Citadel directory - do not type an absolute pathname!) in",
"which to place arriving file transfers that otherwise",
"don't have a home.",

"20",

"21",
"22",
"23",
"24",
"25",
"26",
"27",
"28",
"29",
"30",
"DO NOT re-create files that you wish to keep intact!",
"They will be permanently ERASED if you do so!",
"(Obviously, if this is the first time you are setting up the BBS,",
"then you will want to create all of the files.)",

"31",
"Setup has detected that you currently have data files from a Citadel/UX",
"version 3.2x installation.  The program 'conv_32_40' can upgrade your",
"files to version 4.0x format.",
" Setup will now exit.  Please either run 'conv_32_40' or delete your data",
"files, and run setup again.",

"32",

};


long atol();
struct config config;
int direction;

#ifdef VISUAL_SETUP


void inverse(onoff)
int onoff; {
	if (onoff==1) {
#ifdef A_REVERSE
		attron(A_REVERSE);
#else
		standout();
#endif
		}
	else {
#ifdef A_REVERSE
		attroff(A_REVERSE);
#else
		standend();
#endif
		}
	}


getlin(yp,xp,string,lim)	/* Gets a line from the terminal */
int yp,xp;			/* Where on the screen to start */
char string[];	 		/* Pointer to string buffer */
int lim;			/* Maximum length - if negative, no-show */
{
	int a,b; char flag;

	flag=0;
	if (lim<0) { lim=(0-lim); flag=1; }
	move(yp,xp);
	inverse(1);
	for (a=0; a<lim; ++a) addch('-');
	refresh();
	move(yp,xp);
	for (a=0; a<lim; ++a) addch(' ');
	move(yp,xp);
	printw("%s",string);
GLA:	move(yp,xp+strlen(string));
	refresh();
	a=getch();
	if (a==27) a='x';
	if (a==127) a=8;
#ifdef KEY_LEFT
	if (a==KEY_LEFT) a=8;
#endif
#ifdef KEY_BACKSPACE
	if (a==KEY_BACKSPACE) a=8;
#endif
	if (a==10) a=13;
	if ((a==8)&&(strlen(string)==0)) goto GLA;
	if ((a!=13)&&(a!=8)&&(strlen(string)==lim)) goto GLA;
	if ((a==8)&&(string[0]!=0)) {
		string[strlen(string)-1]=0;
		move(yp,xp+strlen(string));
		addch(' ');
		goto GLA;
		}
	if ((a==13)||(a==10)||(a==KEY_UP)||(a==KEY_DOWN)||(a==4)) {
		inverse(0);
		move(yp,xp);
		for (b=0; b<lim; ++b) addch(' ');
		mvprintw(yp,xp,"%s",string);
		refresh();
		if (a==4) direction = (-3);
		if (a==KEY_UP) direction = (-2);
		else direction = 0;
		return(0);
		}
	b=strlen(string);
	string[b]=a;
	string[b+1]=0;
	if (flag==0) addch(a);
	if (flag==1) addch('*');
	goto GLA;
}
#endif


void title(text)
char *text; {
#ifdef VISUAL_SETUP
	move(7,0);
	printw("%38s",text);
	refresh();
#else
	printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<%s>\n",text);
#endif
	}


void hit_any_key() {
	char junk[5];
#ifdef VISUAL_SETUP
	move(22,20);
	inverse(1);
	printw("Press any key to continue");
	inverse(0);
	refresh();
	getch();
#else
	printf("Press return to continue...\n");
	gets(junk);
#endif
	}

int yesno(question)
char *question; {
	int answer;
	char buf[10];

#ifdef VISUAL_SETUP
	mvprintw(10,0,"%s",question);
	mvprintw(11,0,"Yes/No --> ");
	refresh();
	do {
		answer=getch();
		answer=tolower(answer);
		if (answer=='y') answer=1;
		else if (answer=='n') answer=0;
		} while ((answer<0)||(answer>1));
	mvprintw(10,0,"%80s","");
	mvprintw(11,0,"%80s","");
	refresh();
	return(answer);
#else
	do {
		printf("%s\nYes/No --> ",question);
		gets(buf);
		answer=tolower(buf[0]);
		if (answer=='y') answer=1;
		else if (answer=='n') answer=0;
		} while ((answer<0)||(answer>1));
	return(answer);
#endif
	}


void display_error(text)
char *text; {
#ifdef VISUAL_SETUP
	move(22,0);
	inverse(1);
	printw("%s",text);
	inverse(0);
	refresh();
#else
	printf("\n%s\n",text);
#endif
	}

void dump_access_levels() {
	int a;
#ifdef VISUAL_SETUP
/*
	for (a=0; a<=6; ++a) mvprintw(10+a,0,"%d %s",a,axdefs[a]);
*/
#else
	for (a=0; a<=6; ++a) printf("%d %s\n",a,axdefs[a]);
#endif
	}

void print_setup(msgnum)
int msgnum; {
	int a,b;

	a=0;
	while (atol(setup_text[a]) != msgnum) ++a;
	++a;
#ifdef VISUAL_SETUP
	b = 14;
	move(b,0);
	clrtobot();
	do {
		move(b,0);
		printw("%s",setup_text[a++]);
		++b; if (b>24) b=24;
		} while(atol(setup_text[a])!=(msgnum+1));
#else
	printf("\n\n");
	do {
		printf("%s\n",setup_text[a++]);
		} while(atol(setup_text[a])!=(msgnum+1));
	printf("\n\n");
#endif
	}


void progress(text,curr,cmax)
char *text;
long curr;
long cmax; {
	static long dots_printed;
	long a;

#ifdef VISUAL_SETUP
	if (curr==0) {
		move(5,0);
		for (a=0; a<((80-strlen(text))/2); ++a) addch(' ');
		printw("%s\n",text);
		move(2,0);
		addch(ACS_ULCORNER);
		for (a=0; a<78; ++a) addch(ACS_HLINE);
		addch(ACS_URCORNER);
		move(3,0);
		addch(ACS_VLINE);
		move(3,79);
		addch(ACS_VLINE);
		move(4,0);
		addch(ACS_LLCORNER);
		for (a=0; a<78; ++a) addch(ACS_HLINE);
		addch(ACS_LRCORNER);
		refresh();
		dots_printed = 0;
		move(3,1);
		}
	else if (curr==cmax) {
		mvprintw(5,0,"%80s","");
		mvprintw(2,0,"%80s","");
		mvprintw(3,0,"%80s","");
		mvprintw(4,0,"%80s","");
		refresh();
		}
	else {
		a=(curr * 100) / cmax;
		a=a*78; a=a/100;
		while (dots_printed < a) {
			addch(ACS_BLOCK);
			++dots_printed;
			refresh();
			}
		}
	}
#else
	if (curr==0) {
		printf("%s\n",text);
		printf(".......................................");
		printf(".......................................\r");
		fflush(stdout);
		dots_printed = 0;
		}
	else if (curr==cmax) {
		printf("\r%79s\n","");
		}
	else {
		a=(curr * 100) / cmax;
		a=a*78; a=a/100;
		while (dots_printed < a) {
			printf("*");
			++dots_printed;
			fflush(stdout);
			}
		}
	}
#endif


cre8user() {
	int fd;
	long lbuf;

	/* write null files */
	close(creat("usersupp",0666));
	close(creat("hashtab",0666));

	fd=creat("eternal",0666);
	lbuf = 0L;
	write(fd,&lbuf,sizeof(long));
	close(fd);

	}

cre8msg() {
	struct msgmain msgmain;
	FILE *fp;
	long aa;

	fp=fopen("msgmain","wb");
	for (aa=0; aa<=(MM_FILELEN+100); ++aa) {
		putc(0,fp);
		if ((aa%10000L)==0L)
			progress("Creating message base file...",
				aa,
				(MM_FILELEN+100));
		}
	fclose(fp);

	msgmain.MMlowest=1L;
	msgmain.MMhighest=1L;
	msgmain.MMcurpos=0L;
	msgmain.MMflags=0;

	fp=fopen("MMstructure","wb");
	fwrite((char *)&msgmain,sizeof(struct msgmain),1,fp);
	fclose(fp);
}

cre8room() {
	int a;
	FILE *fp;
	char aaa[50];
	struct quickroom quickroom;
	struct fullroom fullroom;


	/* Load up a blank fullroom structure to be used for all rooms */
	for (a=0; a<MSGSPERRM; ++a) {
		fullroom.FRnum[a]=0L;
		fullroom.FRpos[a]=0L;
		}

	/* Open a new quickroom file */
	fp=fopen("quickroom","wb");
	strcpy(quickroom.QRpasswd,"");
	quickroom.QRroomaide=0L;
	quickroom.QRhighest=0L;
	quickroom.QRgen=1;
	quickroom.QRflags=QR_INUSE;

	/* Create Lobby> Mail> and Aide> */
	strcpy(quickroom.QRname,"Lobby");
	fwrite((char *)&quickroom,sizeof(struct quickroom),1,fp);

	strcpy(quickroom.QRname,"Mail");
	fwrite((char *)&quickroom,sizeof(struct quickroom),1,fp);
	
	strcpy(quickroom.QRname,"Aide");
	fwrite((char *)&quickroom,sizeof(struct quickroom),1,fp);

	/* make the remaining rooms blanks */
	strcpy(quickroom.QRname,"");
	quickroom.QRflags=0;
	for (a=3; a<MAXROOMS; ++a) {
		progress("Creating room index file (quickroom)",
			(long)a,
			(long)MAXROOMS-1
			);
		fwrite((char *)&quickroom,sizeof(struct quickroom),1,fp);
		}
	fclose(fp);

	/* Create directories and fullroom files */
	system("mkdir info 2>/dev/null");
	system("mkdir rooms 2>/dev/null");
	for (a=0; a<MAXROOMS; ++a) {
		progress("Creating message pointer files (fullroom)",
			(long)a,
			(long)MAXROOMS-1
			);
		sprintf(aaa,"./rooms/fullrm%d",a);
		fp=fopen(aaa,"wb");
		fwrite((char *)&fullroom,sizeof(struct fullroom),1,fp);
		fclose(fp);
		}
}

cre8clog() {
	int file,a;
	struct calllog calllog;

	calllog.CLfullname[0]=0;
	calllog.CLtime=0L;
	calllog.CLflags=0;
	a=0;

	file=creat("calllog.pos",0666);
	chmod("calllog.pos",0666);
	write(file,&a,sizeof(int));
	close(file);

	file=creat("calllog",0666);
	chmod("calllog",0666);
	for (a=0; a<CALLLOG; ++a) {
		progress("Creating call log file",
			(long)a,
			(long)CALLLOG-1
			);
		write(file,&calllog,sizeof(struct calllog));
		}
	close(file);
	return(0);
}


int set_str_val(str)
char *str; {
	char buf[128];

#ifdef VISUAL_SETUP
	getlin(7,40,str,39);
#else	
	printf("This is currently set to:\n%s\n",str);
	printf("Enter new value or press return to leave unchanged:\n");
	gets(buf);
	if (strlen(buf)!=0) strcpy(str,buf);
#endif
	}

set_int_val(ip)
int *ip; {
	int a;
	char buf[16];
	sprintf(buf,"%d",(int)*ip);
	set_str_val(buf);
	*ip = atoi(buf);
	}


set_char_val(ip)
char *ip; {
	char buf[16];
	sprintf(buf,"%d",(int)*ip);
	set_str_val(buf);
	*ip = (char)atoi(buf);
	}


set_long_val(ip)
long *ip; {
	char buf[16];
	sprintf(buf,"%ld",*ip);
	set_str_val(buf);
	*ip = atol(buf);
	}


int yesno_s(question) {
	int a;

	a=yesno(question);
	if (a==1) a=yesno("Are you SURE you want to reinitialize this file? ");
	return(a);
	}


/*
 * if we're upgrading from 3.xx to 3.23, we have to set up all of the users
 * to a default 80x24 screen.
 */
void setup_screen_dims() {
	FILE *fp;
	struct usersupp usbuf;
	long numusers,prog;

	fp=fopen("usersupp","r+");
	if (fp==NULL) return;

	fseek(fp,0L,2);
	numusers = ftell(fp) / (long)sizeof(struct usersupp);

	title("Updating user file");
	for (prog=0; prog<numusers; ++prog) {
		fseek(fp,(prog*((long)sizeof(struct usersupp))),0);
		fread((char *)&usbuf,sizeof(struct usersupp),1,fp);
		usbuf.USscreenwidth = 80;
		usbuf.USscreenheight = 24;
		fseek(fp,(prog*((long)sizeof(struct usersupp))),0);
		fwrite((char *)&usbuf,sizeof(struct usersupp),1,fp);
		progress("Updating user file",prog,(numusers-1));
		}

	fclose(fp);
	return;
	}

/*
 * if we're upgrading from 3.xx to 3.23, clear some bits...
 */
void setup_room_bits() {
	FILE *fp;
	struct quickroom qrbuf;
	long prog;

	fp=fopen("quickroom","r+");
	if (fp==NULL) return;

	fseek(fp,0L,2);

	title("Updating room file");
	for (prog=0; prog<MAXROOMS; ++prog) {
		fseek(fp,(prog*((long)sizeof(struct quickroom))),0);
		fread((char *)&qrbuf,sizeof(struct quickroom),1,fp);

		if (qrbuf.QRflags & QR_READONLY)
		   qrbuf.QRflags = (qrbuf.QRflags ^ QR_READONLY);

		fseek(fp,(prog*((long)sizeof(struct quickroom))),0);
		fwrite((char *)&qrbuf,sizeof(struct quickroom),1,fp);
		progress("Updating room file",prog,(MAXROOMS-1));
		}

	fclose(fp);
	return;
	}


void display_value(curr,lin)
int curr,lin; {
#ifdef VISUAL_SETUP
	move(lin,0);
	if ((curr<1)||(curr>19)) {
		printw("%79s ","");
		return;
		}
	printw("%38s",setup_titles[curr]);
	move(lin,40);
	switch(curr) {
		case 1:	printw("%-39s",config.c_nodename);
			break;
		case 2:	printw("%-39s",config.c_fqdn);
			break;
		case 3:	printw("%-39s",config.c_humannode);
			break;
		case 4:	printw("%-39s",config.c_phonenum);
			break;
		case 5:	printw("%-39s",config.c_bbs_city);
			break;
		case 6:	printw("%-39s",config.c_sysadm);
			break;
		case 7:	printw("%-39d",config.c_bbsuid);
			break;
		case 8:	printw("%-39d",config.c_pwcrypt);
			break;
		case 9:	printw("%-39d",config.c_creataide);
			break;
		case 10: printw("%-39d",config.c_sleeping);
			 break;
		case 11: printw("%-39d",config.c_initax);
			 break;
		case 12: printw("%-39d",config.c_regiscall);
			 break;
		case 13: printw("%-39d",config.c_twitdetect);
			 break;
		case 14: printw("%-39s",config.c_twitroom);
			 break;
		case 15: printw("%-39d",config.c_maxsessions);
			 break;
		case 16: printw("%-39s",config.c_moreprompt);
			 break;
		case 17: printw("%-39d",config.c_restrict);
			 break;
		case 18: printw("%-39ld",config.c_msgbase);
			 break;
		case 19: printw("%-39s",config.c_bucket_dir);
			 break;
		}
#endif
	}


edit_value(curr)
int curr; {
 int a;
 
 title(setup_titles[curr]);
 print_setup(curr);
 switch(curr) {

case 1:
	set_str_val(config.c_nodename);
	break;

case 2:
	set_str_val(config.c_fqdn);
	break;

case 3:
	set_str_val(config.c_humannode);
	break;

case 4:
	set_str_val(config.c_phonenum);
	break;

case 5:
	set_str_val(config.c_bbs_city);
	break;

case 6:
	set_str_val(config.c_sysadm);
	break;

case 7:
	set_int_val(&config.c_bbsuid);
	break;

case 8:
	set_int_val(&config.c_pwcrypt);
	break;

case 9:
	set_char_val(&config.c_creataide);
	break;

case 10:
	set_int_val(&config.c_sleeping);
	break;

case 11:
	dump_access_levels();
	set_char_val(&config.c_initax);
	break;

case 12:
	set_char_val(&config.c_regiscall);
	break;

case 13:
	set_char_val(&config.c_twitdetect);
	break;

case 14:
	set_str_val(config.c_twitroom);
	break;

case 15:
	set_int_val(&config.c_maxsessions);
	break;

case 16:
	set_str_val(config.c_moreprompt);
	break;

case 17:
	set_char_val(&config.c_restrict);
	break;

case 18:
	set_long_val(&config.c_msgbase);
	break;

case 19:
	set_str_val(config.c_bucket_dir);
	config.c_bucket_dir[14] = 0;
	for (a=0; a<strlen(config.c_bucket_dir); ++a)
		if (!isalpha(config.c_bucket_dir[a]))
			strcpy(&config.c_bucket_dir[a],
				&config.c_bucket_dir[a+1]);
	break;


 }
}

main() {
	int a;
	int curr;
	char aaa[128];
	FILE *fp;
	int created_msgmain = 0;

	int old_setup_level = 0;
	int new_setup_level = 0;

	chdir(BBSDIR);
	printf("\n\n\n               *** Citadel/UX setup program ***\n\n");

	/*
 	 * what we're going to try to do here is append a whole bunch of
	 * nulls to the citadel.config file, so we can keep the old config
	 * values if they exist, but if the file is missing or from an
	 * earlier version with a shorter config structure, when setup tries
	 * to read the old config parameters, they'll all come up zero.
	 * The length of the config file will be set to what it's supposed
	 * to be when we rewrite it, because we replace the old file with a
	 * completely new copy.  (Neat, eh?)
	 */
	fp=fopen("citadel.config","a");
	if (fp==NULL) {
		fprintf(stderr,"setup: cannot append citadel.config\n");
		exit(1);
		}
	for (a=0; a<sizeof(struct config); ++a) putc(0,fp);
	fclose(fp);

	/* now we re-open it, and read the old or blank configuration */
	fp=fopen("citadel.config","r");
	if (fp==NULL) {
		fprintf(stderr,"setup: cannot open citadel.config\n");
		exit(1);
		}
	fread((char *)&config,sizeof(struct config),1,fp);
	fclose(fp);


	/* set some sample/default values in place of blanks... */
	if (strlen(config.c_nodename)==0)
		strcpy(config.c_nodename,"mysystem");
	if (strlen(config.c_fqdn)==0)
		sprintf(config.c_fqdn,"%s.UUCP",config.c_nodename);
	if (strlen(config.c_humannode)==0)
		strcpy(config.c_humannode,"My System");
	if (strlen(config.c_phonenum)==0)
		strcpy(config.c_phonenum,"US 800 555 1212");
	if (config.c_initax == 0)
		config.c_initax = 1;
	if (config.c_regiscall == 0)
		config.c_regiscall = 1;
	if (strlen(config.c_moreprompt)==0)
		strcpy(config.c_moreprompt,"<more>");
	if (strlen(config.c_twitroom)==0)
		strcpy(config.c_twitroom,"Trashcan");
	if (strlen(config.c_bucket_dir)==0)
		strcpy(config.c_bucket_dir,"bitbucket");
	if (config.c_msgbase == 0L)
		config.c_msgbase = 2000000;


#ifdef VISUAL_SETUP
	initscr();
	raw();
	noecho();
#ifdef ADVANCED_CURSES
	keypad(stdscr,TRUE);
#endif
	clear();
	standout();
	mvprintw(0,0," Citadel/UX Setup ");
	standend();
	move(0,30);
	if (KEY_UP == 16) printw("Previous: Ctrl-P");
	move(0,50);
	if (KEY_DOWN == 14) printw("Next: Ctrl-N");
	else mvprintw(0,30,"Up and down arrow keys are active");
	mvprintw(0,65,"Save: Ctrl-D");
	refresh();
	move(1,0);
	for (a=0; a<80; ++a) addch(ACS_HLINE);
	move(13,0);
	for (a=0; a<80; ++a) addch(ACS_HLINE);

	direction = 0;
	for (curr = 1; curr <= 19; ++curr) {
		for (a=(-5); a<=5; ++a) {
			display_value(curr+a,7+a);
			}
		edit_value(curr);
		curr = curr + direction;
		if (curr < 0) curr = 0;
		if (direction == (-3)) curr = 32767;
		}
#endif

#ifndef VISUAL_SETUP
direction = 0;
for (curr = 1; curr <= 19; ++curr) {
	curr = curr + direction;
	if (curr < 1) curr = 1;
	edit_value(curr);
	}
#endif


	sprintf(aaa,"mkdir %s 2>/dev/null",config.c_bucket_dir);
	system(aaa);

	fp=fopen("citadel.config","w");
	if (fp==NULL) {
		display_error("setup: cannot open citadel.config");
		exit(1);
		}


	/* take care of any updating that is necessary */
	old_setup_level = config.c_setup_level;
	
	if (old_setup_level < 323) {
		setup_screen_dims();
		setup_room_bits();
		config.c_setup_level = 323;
		}



	config.c_setup_level = REV_LEVEL;

	/* now (re-)write the config data to disk */
	fwrite((char *)&config,sizeof(struct config),1,fp);
	fclose(fp);


	if ((config.c_setup_level / 10) == 32) {
#ifdef VISUAL_SETUP
		clear();
		refresh();
#endif
		title("Important Message");
		print_setup(31);
		hit_any_key();
#ifdef VISUAL_SETUP
		clear();
		refresh();
		endwin();
#endif
		exit(0);
		}




#ifdef VISUAL_SETUP
	clear();
#endif
	title("Important Message");
	print_setup(30);
	hit_any_key();

	title("Message base file");
	created_msgmain = 0;
	fp=fopen("msgmain","r");
	if (fp==NULL) {
		cre8msg();
		created_msgmain = 1;
		}
	else {
		fclose(fp);
		if (yesno_s("Create message file?") == 1) {
			cre8msg();
			created_msgmain = 1;
			}
		}

	title("Room file");
	if (created_msgmain == 1) {
		cre8room();
		}
	else {
		fp=fopen("quickroom","r");
		if (fp==NULL) {
			cre8room();
			}
		else {
			fclose(fp);
			if (yesno_s("Create room files?")==1) cre8room();
			}
		}

	title("User file");
	fp=fopen("usersupp","r");
	if (fp==NULL) {
		cre8user();
		}
	else {
		fclose(fp);
		if (yesno_s("Create user file?")==1) cre8user();
		}

	title("Call log");
	a=0;
	fp=fopen("calllog","r");
	if (fp==NULL) {
		cre8clog();
		}
	else {
		fclose(fp);
		if (yesno_s("Create call log?")==1) cre8clog();
		}


	title("Setup finished.");
	hit_any_key();

#ifdef VISUAL_SETUP
	clear();
	refresh();
	endwin();
#endif
	exit(0);
}
