#include <ctype.h>
#include <stdio.h>
#include "citadel.h"

/*
 * struncmp()  -  case-insensitive version of strncmp()
 *                citadel.h will #define a strucmp() based on this
 */
int struncmp(lstr,rstr,len)
char lstr[],rstr[];
int len; {
	int pos = 0;
	char lc,rc;
	while (pos<len) {
		lc=tolower(lstr[pos]);
		rc=tolower(rstr[pos]);
		if ((lc==0)&&(rc==0)) return(0);
		if (lc<rc) return(-1);
		if (lc>rc) return(1);
		pos=pos+1;
		}
	return(0);
	}


/*
 * strproc()  -  make a string 'nice'
 */
strproc(string)
char string[];
{
	int a;

	if (strlen(string)==0) return(0);

	/* Convert non-printable characters to blanks */
	for (a=0; a<strlen(string); ++a) {
		if (string[a]<32) string[a]=32;
		if (string[a]>126) string[a]=32;
		}

	/* Remove leading and trailing blanks */
	while(string[0]<33) strcpy(string,&string[1]);
	while(string[strlen(string)-1]<33) string[strlen(string)-1]=0;

	/* Remove double blanks */
	for (a=0; a<strlen(string); ++a) {
		if ((string[a]==32)&&(string[a+1]==32)) {
			strcpy(&string[a],&string[a+1]);
			a=0;
			}
		}

	/* remove characters which would interfere with the network */
	for (a=0; a<strlen(string); ++a) {
		if (string[a]=='!') strcpy(&string[a],&string[a+1]);
		if (string[a]=='@') strcpy(&string[a],&string[a+1]);
		if (string[a]=='_') strcpy(&string[a],&string[a+1]);
		if (string[a]==',') strcpy(&string[a],&string[a+1]);
		if (string[a]=='%') strcpy(&string[a],&string[a+1]);
		if (string[a]=='|') strcpy(&string[a],&string[a+1]);
		}

	return(0);
	}



/*
 * extract()  -  extract a parameter from a series of "|" separated...
 */
void extract(dest,source,parmnum)
char dest[];
char source[];
int parmnum; {
	char buf[256];
	int count = 0;
	int n;

	n = num_parms(source);

	if (parmnum >= n) {
		strcpy(dest,"");
		return;
		}
	strcpy(buf,source);
	if ( (parmnum == 0) && (n == 1) ) {
		strcpy(dest,buf);
		return;
		}

	while (count++ < parmnum) do {
		strcpy(buf,&buf[1]);
		} while( (strlen(buf)>0) && (buf[0]!='|') );
	if (buf[0]=='|') strcpy(buf,&buf[1]);
	for (count = 0; count<strlen(buf); ++count)
		if (buf[count] == '|') buf[count] = 0;
	strcpy(dest,buf);
	}

/*
 * extract_int()  -  extract an int parm w/o supplying a buffer
 */
int extract_int(source,parmnum)
char *source;
int parmnum; {
	char buf[256];
	
	extract(buf,source,parmnum);
	return(atoi(buf));
	}

/*
 * extract_long()  -  extract an long parm w/o supplying a buffer
 */
long extract_long(source,parmnum)
char *source;
long parmnum; {
	char buf[256];
	
	extract(buf,source,parmnum);
	return(atol(buf));
	}

/*
 * num_parms()  -  discover number of parameters...
 */
int num_parms(source)
char source[]; {
	int a;
	int count = 1;

	for (a=0; a<strlen(source); ++a) 
		if (source[a]=='|') ++count;
	return(count);
	}

/*
 * wrapgetc()  -  same as getc() but can wrap around the EOF
 */
int wrapgetc(fp,wrap)
FILE *fp;
long wrap; {
	int c;

	if (ftell(fp) > wrap)	fseek(fp,0L,0);
	c = getc(fp);
	return(c);
	}

/*
 * wrapgetfield()  -  uses wrapgetc() to fetch a null-terminated string
 */
void wrapgetfield(fp,string,wrap)
FILE *fp;
char string[];
long wrap; {
	int a,b,c;

	strcpy(string,"");
	a=0;
	do {
		b=wrapgetc(fp,wrap);
		if (b<1) {
			string[a]=0;
			return;
			}
		string[a]=b;
		++a;
		} while (b!=0);
	}

/*
 * get a line of text from a file
 * ignores lines starting with #
 */
int getstring(fp,string)
FILE *fp;		
char string[]; {
	int a,c;
	do {
		strcpy(string,"");
		a=0;
		do {
			c=getc(fp);
			if (c<0) {
				string[a]=0;
				return(-1);
				}
			string[a++]=c;
			} while(c!=10);
			string[a-1]=0;
		} while(string[0]=='#');
	return(strlen(string));
	}


/*
 * sort message pointers
 */
void sort_fullroom(frptr)
struct fullroom *frptr; {
	int a,b;
	long hold;
	for (a=MSGSPERRM-2; a>=0; --a) {
		for (b=0; b<=a; ++b) {
			if ((frptr->FRnum[b]) > (frptr->FRnum[b+1])) {
				hold = frptr->FRnum[b];
				frptr->FRnum[b] = frptr->FRnum[b+1];
				frptr->FRnum[b+1] = hold;
				hold = frptr->FRpos[b];
				frptr->FRpos[b] = frptr->FRpos[b+1];
				frptr->FRpos[b+1] = hold;
				}
			}
		}
	}
