/*
 * Citadel/UX "system dependent" stuff
 *
 * By using #ifdef's, we try to make this code run on as many Unix variants
 * as possible.  Wherever possible, if a system-dependent function cannot be
 * used, we use a less efficient/reliable (but portable) construct.
 *
 * THERE IS USUALLY NO NEED TO CHANGE ANYTHING IN THIS FILE.  The configure
 * script will run tests on your system and set the appropriate #define
 * statements in sysdep.h to make the code in this file work properly.
 *
 */

#include <stdio.h>
#include <fcntl.h>

#include "citadel.h"

/* 
 * If we're running on a system which supports file & record locking using the
 * fcntl() system call, then we don't need the flock() stuff.  It doesn't
 * matter whether HAVE_FLOCK is defined or not.  If the F_SETLKW flag exists,
 * then we ignore flock().
 */

#ifdef HAVE_FLOCK
#include <sys/file.h>
#endif


int mms_fd;
int have_lock = 0;




void lock_file(fd,start,len)
	int fd;
	long start;
	long len; {

	int a;

#ifndef HAVE_FLOCK
#ifdef F_SETLKW
	struct flock lockbuf;
#endif
#endif

#ifdef HAVE_FLOCK
	do {
		a=flock(fd,LOCK_EX);
		if (a!=0) sleep(1);
		} while(a!=0);
#else
#ifdef F_SETLKW
	lockbuf.l_type = F_WRLCK;
	lockbuf.l_whence = 0;
	lockbuf.l_start = start;
	lockbuf.l_len = len;
	lockbuf.l_pid = 0L;
	fcntl(mms_fd,F_SETLKW,&lockbuf);
#endif
#endif

	}



void unlock_file(fd,start,len)
	int fd;
	long start;
	long len; {

	int a;

#ifndef HAVE_FLOCK
#ifdef F_SETLKW
	struct flock lockbuf;
#endif
#endif

#ifdef HAVE_FLOCK
	if (flock(fd,LOCK_UN)!=0) fprintf(stderr,"citserver: unlock err\n");
#else
#ifdef F_SETLKW
	lockbuf.l_type = F_UNLCK;
	lockbuf.l_whence = 0;
	lockbuf.l_start = start;
	lockbuf.l_len = len;
	lockbuf.l_pid = 0L;
	fcntl(mms_fd,F_SETLKW,&lockbuf);
#endif
#endif
	}


void lock_msgmain() {
	int a;
	struct msgmain msgmain;


	mms_fd=open("MMstructure",O_RDWR);

#ifdef HAVE_FLOCK
	lock_file(mms_fd,0,2);
#else
#ifdef F_SETLKW
	lock_file(mms_fd,0,2);
#else
	do {
		lseek(mms_fd,0L,0);
		read(mms_fd,&msgmain,sizeof(struct msgmain));
		if (msgmain.MMflags & MM_BUSY) sleep(1);
		} while(msgmain.MMflags & MM_BUSY);

	lseek(mms_fd,0L,0);
	msgmain.MMflags=(msgmain.MMflags|MM_BUSY);
	write(mms_fd,&msgmain,sizeof(struct msgmain));
#endif
#endif
	have_lock = 1;
	}


void unlock_msgmain() {
	struct msgmain msgmain;

	if (have_lock == 0) {
		return;
		}

#ifdef HAVE_FLOCK
	unlock_file(mms_fd,0,2);
#else
#ifdef F_SETLKW
	unlock_file(mms_fd,0,2);
#else
	lseek(mms_fd,0L,0);
	read(mms_fd,&msgmain,sizeof(struct msgmain));
	msgmain.MMflags=msgmain.MMflags & ~MM_BUSY;
	lseek(mms_fd,0L,0);
	write(mms_fd,&msgmain,sizeof(struct msgmain));
#endif
#endif
	close(mms_fd);
	have_lock = 0;
	}

