#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <pwd.h>
#include "citadel.h"
#ifdef USE_SYSLOG
#include <syslog.h>
#endif

struct ucache {
	struct ucache *next;
	char uc_name[32];
	long uc_pos;
	};

struct ucache *ucache = NULL;

extern struct config config;
extern struct usersupp usersupp;
extern struct quickroom quickroom;
extern char curr_user[];
extern int curr_rm;
extern int logged_in;
extern int cs_pos;
extern struct citsession citsession;
extern int nologin;
extern FILE *download_fp;
extern FILE *upload_fp;

char auto_login = 0;

long atol();
void cleanup();

/*
 * pwcrypt()  -  simple password encryption
 */
pwcrypt(text,code)
char text[];
int code; {
	int a;
	for (a=0; a<strlen(text); ++a) text[a]=(text[a]^(((code|128)^a)&0xFF));
	return(0);
	}


/*
 * hash()  -  hash table function for user lookup
 */
int hash(str)
char str[]; {
	int h = 0;
	int i;

	for (i=0; i<strlen(str); ++i) h=h+((i+1)*tolower(str[i]));
	return(h);
	}


/*
 * free_ucache()  -  cleanup routine
 */
void free_ucache() {
	struct ucache *uptr;

	while (ucache!=NULL) {
		uptr = ucache->next;
		free(ucache);
		ucache = uptr;
		}
	}


/*
 * putuser()  -  write user buffer into the correct place on disk
 */
void putuser(usbuf,name)
struct usersupp *usbuf;
char *name; {
	struct ucache *uptr;
	struct usersupp usbuf2;
	int a;
	long pos = (-1L);
	FILE *fp;
	
	for (uptr=ucache; uptr!=NULL; uptr=uptr->next)
		if (!strucmp(uptr->uc_name,name)) pos = uptr->uc_pos;

	/* get the user record so its position goes in the cache */
	if (pos<0L) a = getuser(&usbuf2,name);
	
	for (uptr=ucache; uptr!=NULL; uptr=uptr->next)
		if (!strucmp(uptr->uc_name,name)) pos = uptr->uc_pos;

	/* this is an error condition, but we have no way to report it... */
	if (pos<0L) return;

	fp = fopen("usersupp","r+");
	fseek(fp,pos,0);
	fwrite((char *)usbuf,sizeof(struct usersupp),1,fp);
	fclose(fp);
	}


/*
 * lockuser()  -  lock a user record, if kernel record locking is available
 *                pos = position of record,   op = 1 to lock, 0 to unlock
 *                only one record may be locked at a time
 */
void lockuser(pos,op)
long pos;
int op; {
	static int ulk_fd;

	if (op == 1) {
		ulk_fd=open("usersupp",O_RDWR);
		lock_file(ulk_fd,pos,(long)sizeof(struct usersupp));
		}
	if (op == 0) {
		unlock_file(ulk_fd,pos,(long)sizeof(struct usersupp));
		}

	if (op == 0) {
		close(ulk_fd);
		}
	}


/*
 * getuser()  -  retrieve named user into supplied buffer.
 *               returns 0 on success
 */
int getuser(usbuf,name)
struct usersupp *usbuf;
char *name; {
	FILE *fp,*fp2;
	int uh,fh;
	long count = 0L;
	struct ucache *uptr;

	for (uptr=ucache; uptr!=NULL; uptr=uptr->next) {
		if (!strucmp(ucache->uc_name,name)) {
			fp2=fopen("usersupp","r");
			fseek(fp2,ucache->uc_pos,0);
			fread((struct usersupp *)usbuf,
				sizeof(struct usersupp),1,fp2);
			fclose(fp2);
			if (!strucmp(usbuf->fullname,name)) {
				return(0);
				}
			else {
				strcpy(ucache->uc_name,"___dead___");
				}
			}
		}
	
	uh=hash(name);
	fp=fopen("hashtab","r");
	fp2=fopen("usersupp","r");
	while(fread((char *)&fh,sizeof(int),1,fp)>0) {
		if (uh==fh) {
			fseek(fp2,(count*(long)sizeof(struct usersupp)),0);
			fread((struct usersupp *)usbuf,
				sizeof(struct usersupp),1,fp2);
			if (!strucmp(usbuf->fullname,name)) {
				fclose(fp);
				fclose(fp2);
				uptr = (struct ucache *)
					malloc((long)sizeof(struct ucache));
				if (uptr!=NULL) {
					uptr->next = ucache;
					ucache = uptr;
					strcpy(ucache->uc_name,name);
					ucache->uc_pos = (count * 
						(long)sizeof(struct usersupp));
					}
				return(0);
				}
			}
		++count;
		}
	fclose(fp);
	fclose(fp2);
	return(1);
	}


/*
 * lgetuser()  -  same as getuser() but locks the record
 */
int lgetuser(usbuf,name)
struct usersupp *usbuf;
char *name; {
	int retcode;
	long pos = (-1L);
	struct ucache *uptr;

	retcode = getuser(usbuf,name);
	if (retcode == 0) {
		for (uptr=ucache; uptr!=NULL; uptr=uptr->next)
			if (!strucmp(uptr->uc_name,name)) pos = uptr->uc_pos;
		if (pos >= 0L) {
			lockuser(pos,1);
			}
		}
	return(retcode);
	}


/*
 * putuser()  -  same as putuser() but locks the record
 */
void lputuser(usbuf,name)
struct usersupp *usbuf;
char *name; {
	long pos = (-1L);
	struct ucache *uptr;

	putuser(usbuf,name);
	for (uptr=ucache; uptr!=NULL; uptr=uptr->next)
		if (!strucmp(uptr->uc_name,name)) pos = uptr->uc_pos;
	if (pos >= 0L) {
		lockuser(pos,0);
		}
	}


/*
 * Is the user currently logged in an Aide?
 */
int is_aide() {
	if (usersupp.axlevel >= 6) return(1);
	else return(0);
	}


/*
 * Is the user currently logged in an Aide *or* the room aide for this rm?
 */
int is_room_aide() {
	if ( (usersupp.axlevel >= 6)
		|| (quickroom.QRroomaide == usersupp.eternal) ) return(1);
	else return(0);
	}

/*
 * getuserbynumber()  -  get user by number
 *			 returns 0 if user was found
 */
int getuserbynumber(usbuf,number)
struct usersupp *usbuf;
long number; {
	FILE *fp;
	fp=fopen("./usersupp","r");
	while(fread(usbuf,sizeof(struct usersupp),1,fp)>0) {
		if (usbuf->eternal == number) {
			fclose(fp);
			return(0);
			}
		}
	return(-1);
	}


/*
 * USER cmd
 */
void cmd_user(cmdbuf)
char *cmdbuf; {
	char username[256];
	char autoname[256];
	int found_user = 0;
	struct passwd *p;
	int u,a;

	extract(username,cmdbuf,0);

	if (logged_in) {
		printf("%d Already logged in.\n",ERROR);
		return;
		}

	strproc(username);
	found_user = getuser(&usersupp,username);
	if (found_user != 0) {
		p = (struct passwd *)getpwnam(username);
		if (p!=NULL) {
			strcpy(autoname,p->pw_gecos);
			for (a=0; a<strlen(autoname); ++a)
				if (autoname[a]==',') autoname[a]=0;
			found_user = getuser(&usersupp,autoname);
			}
		}
	if (found_user == 0) {
		if ((nologin) && (usersupp.axlevel < 6)) {
			printf("%d %s: Too many users are already online (maximum is %d)\n",
			ERROR+MAX_SESSIONS_EXCEEDED,
			config.c_nodename,config.c_maxsessions);
			}
		else {
			strcpy(curr_user,usersupp.fullname);
			printf("%d Password required for %s\n",
				MORE_DATA,curr_user);
			}
		}
	else {
		printf("%d %s not found.\n",ERROR,username);
		}
	}



/*
 * session startup code which is common to both cmd_pass() and cmd_newu()
 */
void session_startup() {
	int a;
	struct quickroom qr;
	FILE *fp;

#ifdef USE_SYSLOG
	syslog(LOG_NOTICE,"user <%s> logged in",curr_user);
#endif
	lgetuser(&usersupp,curr_user);
	++usersupp.timescalled;
	time(&usersupp.lastcall);

/* A room's generation number changes each time it is recycled. Users are kept
 * out of private rooms or forget rooms by matching the generation numbers. To
 * avoid an accidental matchup, unmatched numbers are set to -1 here.
 */
	for (a=0; a<MAXROOMS; ++a) {
		getroom(&qr,a);
		if (usersupp.generation[a] != qr.QRgen)
					usersupp.generation[a]=(-1);
		if (usersupp.forget[a] != qr.QRgen)
					usersupp.forget[a]=(-1);
		}

	lputuser(&usersupp,curr_user);

	citsession.cs_pid = getpid();
	strcpy(citsession.cs_name,curr_user);
	fp=fopen("sessions","r+");
	fseek(fp,( ((long)cs_pos)*((long)sizeof(struct citsession)) ),0);
	fwrite((char *)&citsession,sizeof(struct citsession),1,fp);
	fclose(fp);

	printf("%d %s|%d|%d|%d|%u|%ld\n",OK,usersupp.fullname,usersupp.axlevel,
		usersupp.timescalled,usersupp.posted,usersupp.flags,
		usersupp.eternal);
	usergoto(0,0);		/* Enter the lobby */	
	rec_log(CL_LOGIN,curr_user);
	}



/* 
 * misc things to be taken care of when a user is logged out
 */
void logout() {
	FILE *fp;

	logged_in = 0;
	if (download_fp != NULL) {
		fclose(download_fp);
		download_fp = NULL;
		}
	if (upload_fp != NULL) {
		abort_upl();
		}
	citsession.cs_pid = getpid();
	strcpy(citsession.cs_name,"(not logged in)");
	fp=fopen("sessions","r+");
	fseek(fp,( ((long)cs_pos)*((long)sizeof(struct citsession)) ),0);
	fwrite((char *)&citsession,sizeof(struct citsession),1,fp);
	fclose(fp);
	}


void cmd_pass(password)
char *password; {
	int code;
	struct passwd *p;

	if (logged_in) {
		printf("%d Already logged in.\n",ERROR);
		return;
		}
	if (!strcmp(curr_user,"")) {
		printf("%d You must send a name with USER first.\n",ERROR);
		return;
		}
	if (getuser(&usersupp,curr_user)) {
		printf("%d Can't find user record!\n",ERROR+INTERNAL_ERROR);
		return;
		}

	code = (-1);
	if (usersupp.USuid == BBSUID) {
		strproc(password);
		pwcrypt(usersupp.password,config.c_pwcrypt);
		code = strucmp(usersupp.password,password);
		pwcrypt(usersupp.password,config.c_pwcrypt);
		}
	else {
		p = (struct passwd *)getpwuid(usersupp.USuid);
#ifdef ENABLE_AUTOLOGIN
		if (p!=NULL) {
			if (!strcmp(p->pw_passwd,
				(char *)crypt(password,p->pw_passwd)))
					code = 0;
			}
#endif
		}

	if (!code) {
		logged_in = 1;
		session_startup();
		}
	else {
		printf("%d Wrong password.\n",ERROR);
		rec_log(CL_BADPW,curr_user);
		}
	}



/*
 * create_user()  -  back end processing to create a new user
 */
int create_user(username)
char *username; {
	struct usersupp usbuf;
	int a,file;
	FILE *fp;
	long aa;
	struct passwd *p;

	p = (struct passwd *)getpwnam(username);
	strproc(username);
	if ( (!getuser(&usbuf,username)) || (p!=NULL) ) {
		return(ERROR+ALREADY_EXISTS);
		}

	strcpy(curr_user,username);
	strcpy(usersupp.fullname,username);
	logged_in = 1;
	
	a = getuid();
	if ((a==BBSUID) || (a==0)) usersupp.USuid = BBSUID;
	else usersupp.USuid = a;

	for (a=0; a<MAXROOMS; ++a) {
		usersupp.lastseen[a]=0L;
		usersupp.generation[a]=(-1);
		usersupp.forget[a]=(-1);
		}
	for (a=0; a<MAILSLOTS; ++a) {
		usersupp.mailnum[a]=0L;
		usersupp.mailpos[a]=0L;
		}
	strcpy(usersupp.password,"");
	usersupp.flags=US_NEEDVALID|US_LASTOLD|US_DISAPPEAR|US_PAGINATOR;
	usersupp.timescalled=0;
	usersupp.posted=0;
	usersupp.axlevel=INITAX;
	usersupp.USscreenwidth = 80;
	usersupp.USscreenheight = 24;
	time(&usersupp.lastcall);
	strcpy(usersupp.USname,"");
	strcpy(usersupp.USaddr,"");
	strcpy(usersupp.UScity,"");
	strcpy(usersupp.USstate,"");
	strcpy(usersupp.USzip,"");
	strcpy(usersupp.USphone,"");

	file=open("eternal",O_RDWR);	/* get a new user number */
	read(file,&aa,sizeof(long));
	usersupp.eternal=aa++;
	lseek(file,0L,0);
	write(file,&aa,sizeof(long));
	close(file);

	/* add user to userlog and hash table */
	signal(SIGHUP,SIG_IGN);
	fp=fopen("usersupp","a");
	if (ftell(fp)==0L) {
		usersupp.axlevel = 6;
		}
	fwrite((char *)&usersupp,sizeof(struct usersupp),1,fp);
	fclose(fp);
	a=hash(usersupp.fullname);
	fp=fopen("hashtab","a");
	fwrite((char *)&a,sizeof(int),1,fp);
	fclose(fp);
	signal(SIGHUP,cleanup);
	if (getuser(&usersupp,curr_user)) {
		return(ERROR+INTERNAL_ERROR);
		}
	rec_log(CL_NEWUSER,curr_user);
	return(0);
	}




/*
 * cmd_newu()  -  create a new user account
 */
void cmd_newu(username)
char username[]; {
	struct usersupp usbuf;
	int a,file;
	FILE *fp;
	long aa;
	struct passwd *p;

	if (logged_in) {
		printf("%d Already logged in.\n",ERROR);
		return;
		}

	if (nologin) {
		printf("%d %s: Too many users are already online (maximum is %d)\n",
		ERROR+MAX_SESSIONS_EXCEEDED,
		config.c_nodename,config.c_maxsessions);
		}

	a = create_user(username);
	if (a==ERROR+ALREADY_EXISTS) {
		printf("%d '%s' already exists.\n",
			ERROR+ALREADY_EXISTS,username);
		return;
		}
	else if (a==ERROR+INTERNAL_ERROR) {
		printf("%d Internal error - user record disappeared?\n",
			ERROR+INTERNAL_ERROR);
		return;
		}
	else if (a==0) {
		session_startup();
		}
	else {
		printf("%d unknown error\n",ERROR);
		}
	rec_log(CL_NEWUSER,curr_user);
	}



/*
 * automatic login 
 */
void cmd_auto() {
	char username[256];
	struct passwd *p;
	int u,a;

	if (logged_in) {
		printf("%d Already logged in.\n",ERROR);
		return;
		}

#ifdef ENABLE_AUTOLOGIN
	u = getuid();
	if ((u==BBSUID)||(u==0)) {
		printf("%d Auto login only supported when uid is user\n",
			ERROR);
		return;
		}

	p = getpwuid(u);
	if (p==NULL) {
		printf("%d getpwuid() failed.\n",ERROR);
		return;
		}
	strcpy(username,p->pw_gecos);
	for (a=0; a<strlen(username); ++a)
		if (username[a]==',') username[a] = 0;

	if (getuser(&usersupp,username)==0) {
		strcpy(curr_user,username);
		logged_in = 1;
		session_startup();
		return;
		}
	else {
		a=create_user(username);
		if (a==0) session_startup();
		else printf("%d error creating account.\n",a);
		return;
		}
#else
	printf("%d Auto login not supported here.\n",ERROR);
#endif

	}


/*
 * set password
 */
void cmd_setp(new_pw)
char *new_pw; {
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}
	if (usersupp.USuid != BBSUID) {
		printf("%d Not allowed.  Use the 'passwd' command.\n",ERROR);
		return;
		}
	strproc(new_pw);
	lgetuser(&usersupp,curr_user);
	strcpy(usersupp.password,new_pw);
	pwcrypt(usersupp.password,config.c_pwcrypt);
	lputuser(&usersupp,curr_user);
	printf("%d Password changed.\n",OK);
	rec_log(CL_PWCHANGE,curr_user);
	}

/*
 * get user parameters
 */
void cmd_getu() {
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}
	getuser(&usersupp,curr_user);
	printf("%d %d|%d|%d\n",OK,usersupp.USscreenwidth,
		usersupp.USscreenheight,(usersupp.flags & US_USER_SET));
	}

/*
 * set user parameters
 */
void cmd_setu(new_parms)
char *new_parms; {

	if (num_parms(new_parms)!=3) {
		printf("%d Usage error.\n",ERROR);
		return;
		}	
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}
	lgetuser(&usersupp,curr_user);
	usersupp.USscreenwidth = extract_int(new_parms,0);
	usersupp.USscreenheight = extract_int(new_parms,1);
	usersupp.flags = usersupp.flags & (!US_USER_SET);
	usersupp.flags = usersupp.flags | extract_int(new_parms,2);
	lputuser(&usersupp,curr_user);
	printf("%d Ok\n",OK);
	}

/*
 * set last read pointer
 */
void cmd_slrp(new_ptr)
char *new_ptr; {
	long newlr;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (curr_rm < 0) {
		printf("%d No current room.\n",ERROR);
		return;
		}

	if (!struncmp(new_ptr,"highest",7)) {
		newlr = quickroom.QRhighest;
		}
	else {
		newlr = atol(new_ptr);
		}

	lgetuser(&usersupp,curr_user);
	usersupp.lastseen[curr_rm] = newlr;
	lputuser(&usersupp,curr_user);
	printf("%d %ld\n",OK,newlr);
	}


/*
 * INVT and KICK commands
 */
void cmd_invt_kick(iuser,op)
char *iuser;		/* user name */
int op; {		/* 1 = invite, 0 = kick out */
	struct usersupp USscratch;
	char bbb[256];

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (curr_rm < 0) {
		printf("%d No current room.\n",ERROR);
		return;
		}

	if (is_room_aide()==0) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if ( (op==1) && ((quickroom.QRflags&QR_PRIVATE)==0) ) {
		printf("%d Not a private room.\n",ERROR+NOT_HERE);
		return;
		}

	if (lgetuser(&USscratch,iuser)!=0) {
		printf("%d No such user.\n",ERROR);
		return;
		}

	if (op==1) {
		USscratch.generation[curr_rm]=quickroom.QRgen;
		USscratch.forget[curr_rm]=(-1);
		}

	if (op==0) {
		USscratch.generation[curr_rm]=(-1);
		USscratch.forget[curr_rm]=quickroom.QRgen;
		}

	lputuser(&USscratch,iuser);

	/* post a message in Aide> saying what we just did */
	sprintf(bbb,"%s %s %s> by %s",
		iuser,
		((op == 1) ? "invited to" : "kicked out of"),
		quickroom.QRname,
		usersupp.fullname);
	aide_message(bbb);

	if ((op==0)&&((quickroom.QRflags&QR_PRIVATE)==0)) {
		printf("%d Ok. (Not a private room, <Z>ap effect only)\n",OK);
		}
	else {
		printf("%d Ok.\n",OK);
		}
	return;
	}


/*
 * forget (Zap) the current room
 */
void cmd_forg() {
	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (curr_rm < 0) {
		printf("%d No current room.\n",ERROR);
		return;
		}

	if (curr_rm < 3) {
		printf("%d You cannot forget this room.\n",ERROR+NOT_HERE);
		return;
		}

	if (is_aide()) {
		printf("%d Aides cannot forget rooms.\n",ERROR);
		return;
		}

	lgetuser(&usersupp,curr_user);
	usersupp.forget[curr_rm] = quickroom.QRgen;
	usersupp.generation[curr_rm] = (-1);
	lputuser(&usersupp,curr_user);
	printf("%d Ok\n",OK);
	curr_rm = (-1);
	}

/*
 * get next unregistered user
 */
void cmd_gnur() {
	static long holdptr = 0L;
	FILE *fp;
	int a;
	struct usersupp usbuf;
	struct msgmain MMtemp;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (usersupp.axlevel < 6) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	fp=fopen("MMstructure","r");
	fread((char *)&MMtemp,sizeof(struct msgmain),1,fp);
	fclose(fp);
	if ((MMtemp.MMflags&MM_VALID)==0) {
		printf("%d There are no unvalidated users.\n",OK);
		return;
		}

	fp = fopen("usersupp","r");
	if (fp==NULL) {
		printf("%d Can't open userlog.\n",ERROR);
		return;
		}
	fseek(fp,holdptr,0);
	do {
		a=fread((char *)&usbuf,sizeof(struct usersupp),1,fp);
		if ((a==1)&&(usbuf.flags & US_NEEDVALID)) {
			printf("%d %s\n",MORE_DATA,usbuf.fullname);
			holdptr = ftell(fp) - ((long)sizeof(struct usersupp));
			fclose(fp);
			return;
			}
		} while(!feof(fp));
	fclose(fp);
	holdptr = 0L;

	lock_msgmain();
	fp=fopen("MMstructure","r+");
	fread((char *)&MMtemp,sizeof(struct msgmain),1,fp);
	MMtemp.MMflags=MMtemp.MMflags&(~MM_VALID);
	fseek(fp,0L,0);
	fwrite((char *)&MMtemp,sizeof(struct msgmain),1,fp);
	fclose(fp);
	unlock_msgmain();
	printf("%d *** End of registration.\n",OK);
	}


/*
 * get registration info for a user
 */
void cmd_greg(who)
char *who; {
	struct usersupp usbuf;
	int a,b;
	char pbuf[32];

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (!strucmp(who,"_SELF_")) strcpy(who,curr_user);

	if ((usersupp.axlevel < 6) && (strucmp(who,curr_user))) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (getuser(&usbuf,who) != 0) {
		printf("%d '%s' not found.\n",ERROR+NO_SUCH_USER,who);
		return;
		}

	printf("%d %s\n",LISTING_FOLLOWS,usbuf.fullname);
	printf("%ld\n",usbuf.eternal);
	pwcrypt(usbuf.password,PWCRYPT);
	printf("%s\n",usbuf.password);
	printf("%s\n",usbuf.USname);
	printf("%s\n",usbuf.USaddr);
	printf("%s\n%s\n%s\n",
		usbuf.UScity,usbuf.USstate,usbuf.USzip);
	strcpy(pbuf,usbuf.USphone);
	usbuf.USphone[0]=0;
	for (a=0; a<strlen(pbuf); ++a) {
		if ((pbuf[a]>='0')&&(pbuf[a]<='9')) {
			b=strlen(usbuf.USphone);
			usbuf.USphone[b]=pbuf[a];
			usbuf.USphone[b+1]=0;
			}
		}
	while(strlen(usbuf.USphone)<10) {
		strcpy(pbuf,usbuf.USphone);
		strcpy(usbuf.USphone," ");
		strcat(usbuf.USphone,pbuf);
		}

	printf("(%c%c%c) %c%c%c-%c%c%c%c\n",
		usbuf.USphone[0],usbuf.USphone[1],
		usbuf.USphone[2],usbuf.USphone[3],
		usbuf.USphone[4],usbuf.USphone[5],
		usbuf.USphone[6],usbuf.USphone[7],
		usbuf.USphone[8],usbuf.USphone[9]);

	printf("%d\n",usbuf.axlevel);
	printf("000\n");
	}

/*
 * validate a user
 */
void cmd_vali(v_args)
char v_args[]; {
	char user[256];
	int newax;
	struct usersupp userbuf;

	extract(user,v_args,0);
	newax = extract_int(v_args,1);

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	if (usersupp.axlevel < 6) {
		printf("%d Higher access required.\n",
			ERROR+HIGHER_ACCESS_REQUIRED);
		return;
		}

	if (lgetuser(&userbuf,user)!=0) {
		printf("%d '%s' not found.\n",ERROR+NO_SUCH_USER,user);
		return;
		}

	userbuf.axlevel = newax;
	userbuf.flags = (userbuf.flags & ~US_NEEDVALID);

	lputuser(&userbuf,user);
	printf("%d ok\n",OK);
	}



/* 
 *  List users
 */
void cmd_list() {
	FILE *fp;
	struct usersupp usbuf;
	long numusers;

	fp=fopen("usersupp","r");
	fseek(fp,0L,2);
	numusers = (ftell(fp) / (long)sizeof(struct usersupp));
	fseek(fp,0L,0);
	printf("%d %ld\n",LISTING_FOLLOWS,numusers);
	while (fread((char *)&usbuf,sizeof(struct usersupp),1,fp)==1) {
		if ((usersupp.axlevel>=6)||((usbuf.flags&US_UNLISTED)==0)) {
			printf("%s|%d|%ld|%ld|%d|%d|",
				usbuf.fullname,
				usbuf.axlevel,
				usbuf.eternal,
				usbuf.lastcall,
				usbuf.timescalled,
				usbuf.posted);
			pwcrypt(usbuf.password,config.c_pwcrypt);
			if (usersupp.axlevel >= 6) printf("%s",usbuf.password);
			printf("\n");
			}
		}
	printf("000\n");
	fclose(fp);
	}

/*
 * enter registration info
 */
void cmd_regi() {
	int a,b,c;
	FILE *fp;
	char buf[256];
	struct msgmain mm;

	char tmpname[256];
	char tmpaddr[256];
	char tmpcity[256];
	char tmpstate[256];
	char tmpzip[256];
	char tmpphone[256];

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	strcpy(tmpname,"");
	strcpy(tmpaddr,"");
	strcpy(tmpcity,"");
	strcpy(tmpstate,"");
	strcpy(tmpzip,"");
	strcpy(tmpphone,"");

	printf("%d Send registration...\n",SEND_LISTING);
	fflush(stdout);
	a=0;
	while (gets(buf), strcmp(buf,"000")) {
		if (a==0) strcpy(tmpname,buf);
		if (a==1) strcpy(tmpaddr,buf);
		if (a==2) strcpy(tmpcity,buf);
		if (a==3) strcpy(tmpstate,buf);
		if (a==4) {
			for (c=0; c<strlen(buf); ++c) {
				if ((buf[c]>='0')&&(buf[c]<='9')) {
					b=strlen(tmpzip);
					tmpzip[b]=buf[c];
					tmpzip[b+1]=0;
					}
				}
			}
		if (a==5) {
			for (c=0; c<strlen(buf); ++c) {
				if ((buf[c]>='0')&&(buf[c]<='9')) {
					b=strlen(tmpphone);
					tmpphone[b]=buf[c];
					tmpphone[b+1]=0;
					}
				}
			}
		++a;
		}

	tmpname[29]=0;
	tmpaddr[24]=0;
	tmpcity[14]=0;
	tmpstate[2]=0;
	tmpzip[9]=0;
	tmpphone[10]=0;

	lgetuser(&usersupp,curr_user);
	strcpy(usersupp.USname,tmpname);
	strcpy(usersupp.USaddr,tmpaddr);
	strcpy(usersupp.UScity,tmpcity);
	strcpy(usersupp.USstate,tmpstate);
	strcpy(usersupp.USzip,tmpzip);
	strcpy(usersupp.USphone,tmpphone);
	usersupp.flags=(usersupp.flags|US_REGIS|US_NEEDVALID);
	lputuser(&usersupp,curr_user);

	/* set global flag calling for validation */
	lock_msgmain();
	fp=fopen("MMstructure","r+");
	fread((char *)&mm,sizeof(struct msgmain),1,fp);
	mm.MMflags=(mm.MMflags|MM_VALID);
	fseek(fp,0L,0);
	fwrite((char *)&mm,sizeof(struct msgmain),1,fp);
	fclose(fp);
	unlock_msgmain();
	}


/*
 * assorted info we need to check at login
 */
void cmd_chek() {
	int mail = 0;
	int regis = 0;
	int vali = 0;
	struct msgmain msgmain;
	int a,file;

	if (!logged_in) {
		printf("%d Not logged in.\n",ERROR+NOT_LOGGED_IN);
		return;
		}

	getuser(&usersupp,curr_user); /* no lock is needed here */
	if ((REGISCALL!=0)&&((usersupp.flags&US_REGIS)==0)) regis = 1;

	if (usersupp.axlevel >= 6) {
		file=open("MMstructure",O_RDONLY);
		read(file,&msgmain,sizeof(struct msgmain));
		close(file);
		if (msgmain.MMflags&MM_VALID) vali = 1;
		}

	mail=0;				/* check for mail */
	for (a=0; a<MAILSLOTS; ++a)
		if (usersupp.mailnum[a]>usersupp.lastseen[1]) ++mail;

	printf("%d %d|%d|%d\n",OK,mail,regis,vali);
	}


/*
 * check to see if a user exists
 */
void cmd_qusr(who)
char *who; {
	struct usersupp usbuf;

	if (getuser(&usbuf,who) == 0) {
		printf("%d %s\n",OK,usbuf.fullname);
		}
	else {
		printf("%d No such user.\n",ERROR+NO_SUCH_USER);
		}
	}
